@@ -38,6 +38,8 @@
 #include "AIMultiplayer.hxx"
 #include "AITanker.hxx"
 #include "AIWingman.hxx"
+#include "AIGroundVehicle.hxx"
+#include "AIEscort.hxx"
 
 FGAIManager::FGAIManager() {
     _dt = 0.0;
@@ -65,7 +67,7 @@ FGAIManager::init() {
     if (!enabled)
         return;
 
-    wind_from_down_node = fgGetNode("/environment/wind-from-down-fps", true);
+    thermal_lift_node = fgGetNode("/environment/thermal-lift-fps", true);
     wind_from_east_node  = fgGetNode("/environment/wind-from-east-fps",true);
     wind_from_north_node = fgGetNode("/environment/wind-from-north-fps",true);
 
@@ -97,7 +99,7 @@ FGAIManager::postinit() {
             continue;
         }
 
-        SG_LOG(SG_GENERAL, SG_INFO, "loading scenario '" << name << '\'');
+        SG_LOG(SG_GENERAL, SG_ALERT, "loading scenario '" << name << '\'');
         processScenario(name);
         scenarios[name] = true;
     }
@@ -172,7 +174,7 @@ FGAIManager::update(double dt) {
         }
     }
 
-    wind_from_down_node->setDoubleValue( strength ); // for thermals
+    thermal_lift_node->setDoubleValue( strength );  // for thermals
 }
 
 void
@@ -258,6 +260,8 @@ FGAIManager::processThermal( FGAIThermal* thermal ) {
 
 }
 
+
+
 void
 FGAIManager::processScenario( const string &filename ) {
 
@@ -278,8 +282,8 @@ FGAIManager::processScenario( const string &filename ) {
             continue;
         std::string type = scEntry->getStringValue("type", "aircraft");
 
-         if (type == "tanker") { // refueling scenarios
-	    FGAITanker* tanker = new FGAITanker;
+        if (type == "tanker") { // refueling scenarios
+            FGAITanker* tanker = new FGAITanker;
             tanker->readFromScenario(scEntry);
             attach(tanker);
 
@@ -288,7 +292,7 @@ FGAIManager::processScenario( const string &filename ) {
             wingman->readFromScenario(scEntry);
             attach(wingman);
 
-    } else if (type == "aircraft") {
+        } else if (type == "aircraft") {
             FGAIAircraft* aircraft = new FGAIAircraft;
             aircraft->readFromScenario(scEntry);
             attach(aircraft);
@@ -303,6 +307,16 @@ FGAIManager::processScenario( const string &filename ) {
             carrier->readFromScenario(scEntry);
             attach(carrier);
 
+        } else if (type == "groundvehicle") {
+            FGAIGroundVehicle* groundvehicle = new FGAIGroundVehicle;
+            groundvehicle->readFromScenario(scEntry);
+            attach(groundvehicle);
+
+        } else if (type == "escort") {
+            FGAIEscort* escort = new FGAIEscort;
+            escort->readFromScenario(scEntry);
+            attach(escort);
+
         } else if (type == "thunderstorm") {
             FGAIStorm* storm = new FGAIStorm;
             storm->readFromScenario(scEntry);
@@ -322,9 +336,10 @@ FGAIManager::processScenario( const string &filename ) {
             FGAIStatic* aistatic = new FGAIStatic;
             aistatic->readFromScenario(scEntry);
             attach(aistatic);
-
         }
+
     }
+
 }
 
 SGPropertyNode_ptr
@@ -386,8 +401,8 @@ const FGAIBase *
 FGAIManager::calcCollision(double alt, double lat, double lon, double fuse_range)
 {
     // we specify tgt extent (ft) according to the AIObject type
-    double tgt_ht[]     = {0, 50 ,100, 250, 0, 100, 0, 0, 50, 50, 50};
-    double tgt_length[] = {0, 100, 200, 750, 0, 50, 0, 0, 200, 100, 100};
+    double tgt_ht[]     = {0,  50, 100, 250, 0, 100, 0, 0,  50,  50, 20, 100,  50};
+    double tgt_length[] = {0, 100, 200, 750, 0,  50, 0, 0, 200, 100, 40, 200, 100};
     ai_list_iterator ai_list_itr = ai_list.begin();
     ai_list_iterator end = ai_list.end();
 
@@ -397,14 +412,14 @@ FGAIManager::calcCollision(double alt, double lat, double lon, double fuse_range
         tgt_ht[type] += fuse_range;
 
         if (fabs(tgt_alt - alt) > tgt_ht[type] || type == FGAIBase::otBallistic
-                || type == FGAIBase::otStorm || type == FGAIBase::otThermal) {
-            SG_LOG(SG_GENERAL, SG_DEBUG, "AIManager: skipping "
-                << fabs(tgt_alt - alt)
-                << " "
-                << type
-                );
-            ++ai_list_itr;
-            continue;
+            || type == FGAIBase::otStorm || type == FGAIBase::otThermal ) {
+                //SG_LOG(SG_GENERAL, SG_DEBUG, "AIManager: skipping "
+                //    << fabs(tgt_alt - alt)
+                //    << " "
+                //    << type
+                //    );
+                ++ai_list_itr;
+                continue;
         }
 
         double tgt_lat = (*ai_list_itr)->_getLatitude();
@@ -413,14 +428,14 @@ FGAIManager::calcCollision(double alt, double lat, double lon, double fuse_range
 
         double range = calcRange(lat, lon, tgt_lat, tgt_lon);
 
-        SG_LOG(SG_GENERAL, SG_DEBUG, "AIManager:  AI list size "
-            << ai_list.size()
-            << " type " << type
-            << " ID " << id
-            << " range " << range
-            //<< " bearing " << bearing
-            << " alt " << tgt_alt
-            );
+        //SG_LOG(SG_GENERAL, SG_DEBUG, "AIManager:  AI list size "
+        //    << ai_list.size()
+        //    << " type " << type
+        //    << " ID " << id
+        //    << " range " << range
+        //    //<< " bearing " << bearing
+        //    << " alt " << tgt_alt
+        //    );
 
         tgt_length[type] += fuse_range;
 
