@@ -7,7 +7,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include <plib/sg.h>
 #include "hud.hxx"
 
 #ifdef USE_HUD_TextList
@@ -134,9 +133,7 @@ void hud_card::draw(void) //  (HUD_scale * pscale)
             // draw carrot
             drawOneLine(x1, y1, x2, y2);
             drawOneLine(x1, y1, x3, y3);
-            if (snprintf(TextScale, 80, "%3.1f\n", theta) >= 80) {
-                SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-            }
+            sprintf(TextScale,"%3.1f\n", theta);
 
             // draw value
             int l = abs((int)theta);
@@ -388,9 +385,8 @@ void hud_card::draw(void) //  (HUD_scale * pscale)
                                     disp_val = i;
                                 }
 
-                                lenstr = snprintf(TextScale, 80, "%d",
+                                lenstr = sprintf(TextScale, "%d",
                                         float_to_int(disp_val * data_scaling()/*+.5*/));
-
                                 // (int)(disp_val  * data_scaling() +.5));
                                 /* if (((marker_ys - 8) > scrn_rect.top) &&
                                    ((marker_ys + 8) < (height))){ */
@@ -696,7 +692,7 @@ void hud_card::draw(void) //  (HUD_scale * pscale)
                                 }
                                 // printf("disp_val = %d\n", disp_val);
                                 // printf("%d\n", (int)(disp_val  * (double)data_scaling() + 0.5));
-                                lenstr = snprintf(TextScale, 80, "%d",
+                                lenstr = sprintf(TextScale, "%d",
                                         // (int)(disp_val  * data_scaling() +.5));
                                         float_to_int(disp_val * data_scaling()/*+.5*/));
 
@@ -904,9 +900,7 @@ void hud_card::zoomed_scale(int first, int last)
 
             drawOneLine(xstart, ycentre, xstart - 5.0, ycentre); //centre tick
 
-            if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre] * data_scaling())) >= 80) {
-                SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-            }
+            sprintf(TextScale,"%3.0f\n",(float)(data[centre] * data_scaling()));
 
             if (!huds_notext(options))
                 textString(x, ycentre, TextScale, 0);
@@ -924,16 +918,12 @@ void hud_card::zoomed_scale(int first, int last)
                 drawOneLine(xstart, yfirst, xstart - 5.0, yfirst);
                 drawOneLine(xstart, ysecond, xstart - 5.0, ysecond);
 
-                if (snprintf(TextScale,80, "%3.0f\n",(float)(data[centre - i - 1] * data_scaling())) >= 80) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-                }
+                sprintf(TextScale,"%3.0f\n",(float)(data[centre - i - 1] * data_scaling()));
 
                 if (!huds_notext(options))
                     textString (x, yfirst, TextScale, 0);
 
-                if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre + i + 1] * data_scaling())) >= 80) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-                }
+                sprintf(TextScale,"%3.0f\n",(float)(data[centre + i + 1] * data_scaling()));
 
                 if (!huds_notext(options))
                     textString (x, ysecond, TextScale, 0);
@@ -961,9 +951,7 @@ void hud_card::zoomed_scale(int first, int last)
 
             drawOneLine(xstart, ycentre, xstart + 5.0, ycentre); //centre tick
 
-            if (snprintf(TextScale,80, "%3.0f\n",(float)(data[centre] * data_scaling())) >= 80) {
-                SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-            }
+            sprintf(TextScale,"%3.0f\n",(float)(data[centre] * data_scaling()));
 
             if (!huds_notext(options))
                 textString(w, ycentre, TextScale, 0);
@@ -981,16 +969,12 @@ void hud_card::zoomed_scale(int first, int last)
                 drawOneLine(xstart, yfirst, xstart + 5.0, yfirst);
                 drawOneLine(xstart, ysecond, xstart + 5.0, ysecond);
 
-                if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre - i - 1] * data_scaling())) >= 80) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-                }
+                sprintf(TextScale,"%3.0f\n",(float)(data[centre - i - 1] * data_scaling()));
 
                 if (!huds_notext(options))
                     textString(w, yfirst, TextScale, 0);
 
-                if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre + i + 1] * data_scaling())) >= 80) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-                }
+                sprintf(TextScale,"%3.0f\n",(float)(data[centre + i + 1] * data_scaling()));
 
                 if (!huds_notext(options))
                     textString(w, ysecond, TextScale, 0);
@@ -1079,9 +1063,7 @@ void hud_card::zoomed_scale(int first, int last)
             ystart = h;
             drawOneLine(xcentre, ystart, xcentre, ystart - 5.0); //centre tick
 
-            if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre] * data_scaling())) >= 80) { 
-                SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-            }
+            sprintf(TextScale,"%3.0f\n",(float)(data[centre] * data_scaling()));
 
             if (!huds_notext(options))
                 textString (xcentre - 10.0, y, TextScale, 0);
@@ -1099,16 +1081,12 @@ void hud_card::zoomed_scale(int first, int last)
                 drawOneLine(xfirst, ystart, xfirst,  ystart - 5.0);
                 drawOneLine(xsecond, ystart, xsecond, ystart - 5.0);
 
-                if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre - i - 1] * data_scaling())) >= 80) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-                }
+                sprintf(TextScale,"%3.0f\n",(float)(data[centre - i - 1] * data_scaling()));
 
                 if (!huds_notext(options))
                     textString (xfirst - 10.0, y, TextScale, 0);
 
-                if (snprintf(TextScale,80, "%3.0f\n",(float)(data[centre + i + 1] * data_scaling())) >= 80) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-                }
+                sprintf(TextScale,"%3.0f\n",(float)(data[centre + i + 1] * data_scaling()));
 
                 if (!huds_notext(options))
                     textString (xsecond - 10.0, y, TextScale, 0);
@@ -1137,9 +1115,7 @@ void hud_card::zoomed_scale(int first, int last)
             //drawOneLine(xstart, yfirst,  xstart - 5.0, yfirst);
             drawOneLine(xcentre, ystart, xcentre, ystart + 5.0); //centre tick
 
-            if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre] * data_scaling())) >= 80) {
-                SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-            }
+            sprintf(TextScale,"%3.0f\n",(float)(data[centre] * data_scaling()));
 
             if (!huds_notext(options))
                 textString (xcentre - 10.0, h, TextScale, 0);
@@ -1157,16 +1133,12 @@ void hud_card::zoomed_scale(int first, int last)
                 drawOneLine(xfirst, ystart, xfirst, ystart + 5.0);
                 drawOneLine(xsecond, ystart, xsecond, ystart + 5.0);
 
-                if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre - i - 1] * data_scaling())) >= 80) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-                }
+                sprintf(TextScale,"%3.0f\n",(float)(data[centre - i - 1] * data_scaling()));
 
                 if (!huds_notext(options))
                     textString (xfirst - 10.0, h, TextScale, 0);
 
-                if (snprintf(TextScale, 80, "%3.0f\n",(float)(data[centre + i + 1] * data_scaling())) >= 80) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-                }
+                sprintf(TextScale,"%3.0f\n",(float)(data[centre + i + 1] * data_scaling()));
 
                 if (!huds_notext(options))
                     textString (xsecond - 10.0, h, TextScale, 0);
