@@ -5,7 +5,7 @@
  Date started: 11/17/98
  Purpose:      Schedules and runs the model routines.
 
- ------------- Copyright (C) 1999  Jon S. Berndt (jsb@hal-pc.org) -------------
+ ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free Software
@@ -43,29 +43,32 @@ INCLUDES
 
 #include "FGFDMExec.h"
 #include "FGState.h"
-#include <models/FGAtmosphere.h>
-#include <models/atmosphere/FGMSIS.h>
-#include <models/atmosphere/FGMars.h>
-#include <models/FGFCS.h>
-#include <models/FGPropulsion.h>
-#include <models/FGMassBalance.h>
-#include <models/FGGroundReactions.h>
-#include <models/FGExternalReactions.h>
-#include <models/FGBuoyantForces.h>
-#include <models/FGAerodynamics.h>
-#include <models/FGInertial.h>
-#include <models/FGAircraft.h>
-#include <models/FGPropagate.h>
-#include <models/FGAuxiliary.h>
-#include <models/FGInput.h>
-#include <models/FGOutput.h>
-#include <initialization/FGInitialCondition.h>
-//#include <initialization/FGTrimAnalysis.h> // Remove until later
-#include <input_output/FGPropertyManager.h>
-#include <input_output/FGScript.h>
+#include "models/FGAtmosphere.h"
+#include "models/atmosphere/FGMSIS.h"
+#include "models/atmosphere/FGMars.h"
+#include "models/FGFCS.h"
+#include "models/FGPropulsion.h"
+#include "models/FGMassBalance.h"
+#include "models/FGGroundReactions.h"
+#include "models/FGExternalReactions.h"
+#include "models/FGBuoyantForces.h"
+#include "models/FGAerodynamics.h"
+#include "models/FGInertial.h"
+#include "models/FGAircraft.h"
+#include "models/FGPropagate.h"
+#include "models/FGAuxiliary.h"
+#include "models/FGInput.h"
+#include "models/FGOutput.h"
+#include "initialization/FGInitialCondition.h"
+//#include "initialization/FGTrimAnalysis.h" // Remove until later
+#include "input_output/FGPropertyManager.h"
+#include "input_output/FGScript.h"
 
 #include <iostream>
 #include <iterator>
+#include <cstdlib>
+
+using namespace std;
 
 namespace JSBSim {
 
@@ -90,9 +93,9 @@ void checkTied ( FGPropertyManager *node )
 
   for (int i=0; i<N; i++) {
     if (node->getChild(i)->nChildren() ) {
-//      cout << "Untieing " << node->getChild(i)->getName() << " property branch." << endl;
       checkTied( (FGPropertyManager*)node->getChild(i) );
-    } else if ( node->getChild(i)->isTied() ) {
+    }
+    if ( node->getChild(i)->isTied() ) {
       name = ((FGPropertyManager*)node->getChild(i))->GetFullyQualifiedName();
       node->Untie(name);
     }
@@ -106,7 +109,6 @@ FGFDMExec::FGFDMExec(FGPropertyManager* root) : Root(root)
 {
 
   Frame           = 0;
-  FirstModel      = 0;
   Error           = 0;
   GroundCallback  = 0;
   State           = 0;
@@ -128,19 +130,12 @@ FGFDMExec::FGFDMExec(FGPropertyManager* root) : Root(root)
   Script          = 0;
 
   modelLoaded = false;
-  IsSlave = false;
+  IsChild = false;
   holding = false;
   Terminate = false;
 
-  // Multiple FDM's are stopped for now.  We need to ensure that
-  // the "user" instance always gets the zeroeth instance number,
-  // because there may be instruments or scripts tied to properties
-  // in the jsbsim[0] node.
-  // ToDo: it could be that when JSBSim is reset and a new FDM is wanted, that
-  // process might try setting FDMctr = 0. Then the line below would not need
-  // to be commented out.
-  IdFDM = FDMctr;
-  //FDMctr++;
+  IdFDM = FDMctr; // The main (parent) JSBSim instance is always the "zeroth"
+  FDMctr++;       // instance. "child" instances are loaded last.
 
   try {
     char* num = getenv("JSBSIM_DEBUG");
@@ -172,6 +167,7 @@ FGFDMExec::FGFDMExec(FGPropertyManager* root) : Root(root)
   typedef int (FGFDMExec::*iPMF)(void) const;
 //  instance->Tie("simulation/do_trim_analysis", this, (iPMF)0, &FGFDMExec::DoTrimAnalysis);
   instance->Tie("simulation/do_simple_trim", this, (iPMF)0, &FGFDMExec::DoTrim);
+  instance->Tie("simulation/reset", this, (iPMF)0, &FGFDMExec::ResetToInitialConditions);
   instance->Tie("simulation/terminate", (int *)&Terminate);
   Constructing = false;
 }
@@ -188,10 +184,12 @@ FGFDMExec::~FGFDMExec()
     cout << "Caught error: " << msg << endl;
   }
 
-  for (unsigned int i=1; i<SlaveFDMList.size(); i++) delete SlaveFDMList[i]->exec;
-  SlaveFDMList.clear();
+  for (unsigned int i=1; i<ChildFDMList.size(); i++) delete ChildFDMList[i]->exec;
+  ChildFDMList.clear();
 
-  //ToDo remove property catalog.
+  PropertyCatalog.clear();
+
+  FDMctr--;
 
   Debug(1);
 }
@@ -223,30 +221,11 @@ bool FGFDMExec::Allocate(void)
                                        // class needs valid pointers to the above
                                        // model classes
 
-  // Initialize models so they can communicate with each other
-
-  Atmosphere->InitModel();
-  FCS->InitModel();
-  Propulsion->InitModel();
-  MassBalance->InitModel();
-  Aerodynamics->InitModel();
-  Inertial->InitModel();
-  GroundReactions->InitModel();
-  ExternalReactions->InitModel();
-  BuoyantForces->InitModel();
-  Aircraft->InitModel();
-  Propagate->InitModel();
-  Auxiliary->InitModel();
-  Input->InitModel();
-
-  IC = new FGInitialCondition(this);
-
   // Schedule a model. The second arg (the integer) is the pass number. For
-  // instance, the atmosphere model could get executed every fifth pass it is called
-  // by the executive. IC and Trim objects are NOT scheduled.
-
+  // instance, the atmosphere model could get executed every fifth pass it is called.
+  
   Schedule(Input,           1);
-  Schedule(Atmosphere,      1);
+  Schedule(Atmosphere,      30);
   Schedule(FCS,             1);
   Schedule(Propulsion,      1);
   Schedule(MassBalance,     1);
@@ -259,6 +238,13 @@ bool FGFDMExec::Allocate(void)
   Schedule(Propagate,       1);
   Schedule(Auxiliary,       1);
 
+  // Initialize models so they can communicate with each other
+
+  vector <FGModel*>::iterator it;
+  for (it = Models.begin(); it != Models.end(); ++it) (*it)->InitModel();
+
+  IC = new FGInitialCondition(this);
+
   modelLoaded = false;
 
   return result;
@@ -292,7 +278,6 @@ bool FGFDMExec::DeAllocate(void)
 
   delete GroundCallback;
 
-  FirstModel  = 0L;
   Error       = 0;
 
   State           = 0;
@@ -311,35 +296,18 @@ bool FGFDMExec::DeAllocate(void)
   Auxiliary       = 0;
   Script          = 0;
 
+  Models.clear();
+
   modelLoaded = false;
   return modelLoaded;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-int FGFDMExec::Schedule(FGModel* model, int rate)
+void FGFDMExec::Schedule(FGModel* model, int rate)
 {
-  FGModel* model_iterator;
-
-  model_iterator = FirstModel;
-
-  if (model_iterator == 0L) {                  // this is the first model
-
-    FirstModel = model;
-    FirstModel->NextModel = 0L;
-    FirstModel->SetRate(rate);
-
-  } else {                                     // subsequent model
-
-    while (model_iterator->NextModel != 0L) {
-      model_iterator = model_iterator->NextModel;
-    }
-    model_iterator->NextModel = model;
-    model_iterator->NextModel->SetRate(rate);
-
-  }
-
-  return 0;
+  model->SetRate(rate);
+  Models.push_back(model);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -347,26 +315,19 @@ int FGFDMExec::Schedule(FGModel* model, int rate)
 bool FGFDMExec::Run(void)
 {
   bool success=true;
-  FGModel* model_iterator;
-
-  model_iterator = FirstModel;
-  if (model_iterator == 0L) return false;
 
   Debug(2);
 
-  for (unsigned int i=1; i<SlaveFDMList.size(); i++) {
-//    SlaveFDMList[i]->exec->State->Initialize(); // Transfer state to the slave FDM
-//    SlaveFDMList[i]->exec->Run();
+  for (unsigned int i=1; i<ChildFDMList.size(); i++) {
+    ChildFDMList[i]->AssignState(Propagate); // Transfer state to the child FDM
+    ChildFDMList[i]->Run();
   }
 
-  // returns true if success
-  // false if complete
+  // returns true if success, false if complete
   if (Script != 0 && !State->IntegrationSuspended()) success = Script->RunScript();
 
-  while (model_iterator != 0L) {
-    model_iterator->Run();
-    model_iterator = model_iterator->NextModel;
-  }
+  vector <FGModel*>::iterator it;
+  for (it = Models.begin(); it != Models.end(); ++it) (*it)->Run();
 
   Frame++;
   if (!Holding()) State->IncrTime();
@@ -388,19 +349,30 @@ bool FGFDMExec::RunIC(void)
   return true;
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+//
+// A private, internal function call for Tie-ing to a property, so it needs an
+// argument.
+
+void FGFDMExec::ResetToInitialConditions(int mode)
+{
+  if (mode == 1) {
+    for (unsigned int i=0; i<Outputs.size(); i++) {
+      Outputs[i]->SetStartNewFile(true); 
+    }
+  }
+  
+  ResetToInitialConditions();
+}
+
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 void FGFDMExec::ResetToInitialConditions(void)
 {
-  FGModel* model_iterator;
+  if (Constructing) return;
 
-  model_iterator = FirstModel;
-  if (model_iterator == 0L) return;
-
-  while (model_iterator != 0L) {
-    model_iterator->InitModel();
-    model_iterator = model_iterator->NextModel;
-  }
+  vector <FGModel*>::iterator it;
+  for (it = Models.begin(); it != Models.end(); ++it) (*it)->InitModel();
 
   RunIC();
   if (Script) Script->ResetEvents();
@@ -436,8 +408,8 @@ vector <string> FGFDMExec::EnumerateFDMs(void)
 
   FDMList.push_back(Aircraft->GetAircraftName());
 
-  for (unsigned int i=1; i<SlaveFDMList.size(); i++) {
-    FDMList.push_back(SlaveFDMList[i]->exec->GetAircraft()->GetAircraftName());
+  for (unsigned int i=1; i<ChildFDMList.size(); i++) {
+    FDMList.push_back(ChildFDMList[i]->exec->GetAircraft()->GetAircraftName());
   }
 
   return FDMList;
@@ -494,59 +466,197 @@ bool FGFDMExec::LoadModel(string model, bool addModelToPath)
     Allocate();
   }
 
+  int saved_debug_lvl = debug_lvl;
+
   document = LoadXMLDocument(aircraftCfgFileName); // "document" is a class member
   if (document) {
+    if (IsChild) debug_lvl = 0;
+
     ReadPrologue(document);
-    element = document->GetElement();
 
-    result = true;
-    while (element && result) {
-      string element_name = element->GetName();
-      if (element_name == "fileheader" )           result = ReadFileHeader(element);
-      else if (element_name == "slave")            result = ReadSlave(element);
-      else if (element_name == "metrics")          result = Aircraft->Load(element);
-      else if (element_name == "mass_balance")     result = MassBalance->Load(element);
-      else if (element_name == "ground_reactions") result = GroundReactions->Load(element);
-      else if (element_name == "external_reactions") result = ExternalReactions->Load(element);
-      else if (element_name == "buoyant_forces")   result = BuoyantForces->Load(element);
-      else if (element_name == "propulsion")       result = Propulsion->Load(element);
-      else if (element_name == "system")           result = FCS->Load(element,
-                                                            FGFCS::stSystem);
-      else if (element_name == "autopilot")        result = FCS->Load(element,
-                                                            FGFCS::stAutoPilot);
-      else if (element_name == "flight_control")   result = FCS->Load(element,
-                                                            FGFCS::stFCS);
-      else if (element_name == "aerodynamics")     result = Aerodynamics->Load(element);
-      else if (element_name == "input")            result = Input->Load(element);
-      else if (element_name == "output")           {
-          FGOutput* Output = new FGOutput(this);
-          Output->InitModel();
-          Schedule(Output,       1);
-          result = Output->Load(element);
-          Outputs.push_back(Output);
+    if (IsChild) debug_lvl = saved_debug_lvl;
+
+    // Process the fileheader element in the aircraft config file. This element is OPTIONAL.
+    element = document->FindElement("fileheader");
+    if (element) {
+      result = ReadFileHeader(element);
+      if (!result) {
+        cerr << endl << "Aircraft fileheader element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    }
+
+    if (IsChild) debug_lvl = 0;
+
+    // Process the metrics element. This element is REQUIRED.
+    element = document->FindElement("metrics");
+    if (element) {
+      result = Aircraft->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft metrics element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    } else {
+      cerr << endl << "No metrics element was found in the aircraft config file." << endl;
+      return false;
+    }
+
+    // Process the mass_balance element. This element is REQUIRED.
+    element = document->FindElement("mass_balance");
+    if (element) {
+      result = MassBalance->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft mass_balance element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    } else {
+      cerr << endl << "No mass_balance element was found in the aircraft config file." << endl;
+      return false;
+    }
+
+    // Process the ground_reactions element. This element is REQUIRED.
+    element = document->FindElement("ground_reactions");
+    if (element) {
+      result = GroundReactions->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft ground_reactions element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    } else {
+      cerr << endl << "No ground_reactions element was found in the aircraft config file." << endl;
+      return false;
+    }
+
+    // Process the external_reactions element. This element is OPTIONAL.
+    element = document->FindElement("external_reactions");
+    if (element) {
+      result = ExternalReactions->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft external_reactions element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    }
+
+    // Process the buoyant_forces element. This element is OPTIONAL.
+    element = document->FindElement("buoyant_forces");
+    if (element) {
+      result = BuoyantForces->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft buoyant_forces element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    }
+
+    // Process the propulsion element. This element is OPTIONAL.
+    element = document->FindElement("propulsion");
+    if (element) {
+      result = Propulsion->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft propulsion element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    }
+
+    // Process the system element[s]. This element is OPTIONAL, and there may be more than one.
+    element = document->FindElement("system");
+    while (element) {
+      result = FCS->Load(element, FGFCS::stSystem);
+      if (!result) {
+        cerr << endl << "Aircraft system element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+      element = document->FindNextElement("system");
+    }
+
+    // Process the autopilot element. This element is OPTIONAL.
+    element = document->FindElement("autopilot");
+    if (element) {
+      result = FCS->Load(element, FGFCS::stAutoPilot);
+      if (!result) {
+        cerr << endl << "Aircraft autopilot element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    }
+
+    // Process the flight_control element. This element is OPTIONAL.
+    element = document->FindElement("flight_control");
+    if (element) {
+      result = FCS->Load(element, FGFCS::stFCS);
+      if (!result) {
+        cerr << endl << "Aircraft flight_control element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    }
+
+    // Process the aerodynamics element. This element is OPTIONAL, but almost always expected.
+    element = document->FindElement("aerodynamics");
+    if (element) {
+      result = Aerodynamics->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft aerodynamics element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    } else {
+      cerr << endl << "No expected aerodynamics element was found in the aircraft config file." << endl;
+    }
+
+    // Process the input element. This element is OPTIONAL.
+    element = document->FindElement("input");
+    if (element) {
+      result = Input->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft input element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      }
+    }
+
+    // Process the output element[s]. This element is OPTIONAL, and there may be more than one.
+    unsigned int idx=0;
+    typedef int (FGOutput::*iOPMF)(void) const;
+    element = document->FindElement("output");
+    while (element) {
+      if (debug_lvl > 0) cout << endl << "  Output data set: " << idx << "  ";
+      FGOutput* Output = new FGOutput(this);
+      Output->InitModel();
+      Schedule(Output, 1);
+      result = Output->Load(element);
+      if (!result) {
+        cerr << endl << "Aircraft output element has problems in file " << aircraftCfgFileName << endl;
+        return result;
+      } else {
+        Outputs.push_back(Output);
+        string outputProp = CreateIndexedPropertyName("simulation/output",idx);
+        instance->Tie(outputProp+"/log_rate_hz", Output, (iOPMF)0, &FGOutput::SetRate);
+        idx++;
+      }
+      element = document->FindNextElement("output");
+    }
+
+    // Lastly, process the child element. This element is OPTIONAL - and NOT YET SUPPORTED.
+    element = document->FindElement("child");
+    if (element) {
+      result = ReadChild(element);
+      if (!result) {
+        cerr << endl << "Aircraft child element has problems in file " << aircraftCfgFileName << endl;
+        return result;
       }
-      else {
-        cerr << "Found unexpected subsystem: " << element_name << ", exiting." << endl;
-        result = false;
-        break;
-      }
-      element = document->GetNextElement();
     }
-  } else {
-    cerr << fgred
-         << "  JSBSim failed to load aircraft model."
-         << fgdef << endl;
-    return false;
-  }
 
-  if (result) {
     modelLoaded = true;
-    Debug(3);
+
+    if (debug_lvl > 0) {
+      cout << endl << fgblue << highint
+           << "End of vehicle configuration loading." << endl
+           << "-------------------------------------------------------------------------------"
+           << reset << endl;
+    }
+    
+    if (IsChild) debug_lvl = saved_debug_lvl;
+
   } else {
     cerr << fgred
-         << "  JSBSim failed to load properly."
+         << "  JSBSim failed to open the configuration file: " << aircraftCfgFileName
          << fgdef << endl;
-    return false;
   }
 
   struct PropertyCatalogStructure masterPCS;
@@ -564,14 +674,17 @@ void FGFDMExec::BuildPropertyCatalog(struct PropertyCatalogStructure* pcs)
 {
   struct PropertyCatalogStructure* pcsNew = new struct PropertyCatalogStructure;
   int node_idx = 0;
-  char int_buf[10];
 
-  for (unsigned int i=0; i<pcs->node->nChildren(); i++) {
+  for (int i=0; i<pcs->node->nChildren(); i++) {
     pcsNew->base_string = pcs->base_string + "/" + pcs->node->getChild(i)->getName();
     node_idx = pcs->node->getChild(i)->getIndex();
-    sprintf(int_buf, "[%d]", node_idx);
-    if (node_idx != 0) pcsNew->base_string += string(int_buf);
+    if (node_idx != 0) {
+      pcsNew->base_string = CreateIndexedPropertyName(pcsNew->base_string, node_idx);
+    }
     if (pcs->node->getChild(i)->nChildren() == 0) {
+      if (pcsNew->base_string.substr(0,11) == string("/fdm/jsbsim")) {
+        pcsNew->base_string = pcsNew->base_string.erase(0,12);
+      }
       PropertyCatalog.push_back(pcsNew->base_string);
     } else {
       pcsNew->node = (FGPropertyManager*)pcs->node->getChild(i);
@@ -611,16 +724,20 @@ bool FGFDMExec::ReadFileHeader(Element* el)
 {
   bool result = true; // true for success
 
-  if (debug_lvl & ~1) return result;
+  if (debug_lvl == 0) return result;
 
+  if (IsChild) {
+    cout << endl <<highint << fgblue << "Reading child model: " << IdFDM << reset << endl << endl;
+  }
+
+  if (el->FindElement("description"))
+    cout << "  Description:   " << el->FindElement("description")->GetDataLine() << endl;
   if (el->FindElement("author"))
     cout << "  Model Author:  " << el->FindElement("author")->GetDataLine() << endl;
   if (el->FindElement("filecreationdate"))
     cout << "  Creation Date: " << el->FindElement("filecreationdate")->GetDataLine() << endl;
   if (el->FindElement("version"))
     cout << "  Version:       " << el->FindElement("version")->GetDataLine() << endl;
-  if (el->FindElement("description"))
-    cout << "  Description:   " << el->FindElement("description")->GetDataLine() << endl;
 
   return result;
 }
@@ -687,54 +804,51 @@ bool FGFDMExec::ReadPrologue(Element* el) // el for ReadPrologue is the document
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-bool FGFDMExec::ReadSlave(Element* el)
+bool FGFDMExec::ReadChild(Element* el)
 {
-  // Add a new slaveData object to the slave FDM list
-  // Populate that slaveData element with a new FDMExec object
-  // Set the IsSlave flag for that FDMExec object
+  // Add a new childData object to the child FDM list
+  // Populate that childData element with a new FDMExec object
+  // Set the IsChild flag for that FDMExec object
   // Get the aircraft name
-  // set debug level to print out no additional data for slave objects
+  // set debug level to print out no additional data for child objects
   // Load the model given the aircraft name
   // reset debug level to prior setting
 
-  int saved_debug_lvl = debug_lvl;
   string token;
 
-  SlaveFDMList.push_back(new slaveData);
-  SlaveFDMList.back()->exec = new FGFDMExec();
-  SlaveFDMList.back()->exec->SetSlave(true);
-/*
-  string AircraftName = AC_cfg->GetValue("file");
+  struct childData* child = new childData;
 
-  debug_lvl = 0;                 // turn off debug output for slave vehicle
+  child->exec = new FGFDMExec();
+  child->exec->SetChild(true);
 
-  SlaveFDMList.back()->exec->SetAircraftPath( AircraftPath );
-  SlaveFDMList.back()->exec->SetEnginePath( EnginePath );
-  SlaveFDMList.back()->exec->SetSystemsPath( SystemsPath );
-  SlaveFDMList.back()->exec->LoadModel(AircraftName);
-  debug_lvl = saved_debug_lvl;   // turn debug output back on for master vehicle
+  string childAircraft = el->GetAttributeValue("name");
+  string sMated = el->GetAttributeValue("mated");
+  if (sMated == "false") child->mated = false; // child objects are mated by default.
+  string sInternal = el->GetAttributeValue("internal");
+  if (sInternal == "true") child->internal = true; // child objects are external by default.
 
-  AC_cfg->GetNextConfigLine();
-  while ((token = AC_cfg->GetValue()) != string("/SLAVE")) {
-    *AC_cfg >> token;
-    if      (token == "xloc")  { *AC_cfg >> SlaveFDMList.back()->x;    }
-    else if (token == "yloc")  { *AC_cfg >> SlaveFDMList.back()->y;    }
-    else if (token == "zloc")  { *AC_cfg >> SlaveFDMList.back()->z;    }
-    else if (token == "pitch") { *AC_cfg >> SlaveFDMList.back()->pitch;}
-    else if (token == "yaw")   { *AC_cfg >> SlaveFDMList.back()->yaw;  }
-    else if (token == "roll")  { *AC_cfg >> SlaveFDMList.back()->roll;  }
-    else cerr << "Unknown identifier: " << token << " in slave vehicle definition" << endl;
+  child->exec->SetAircraftPath( AircraftPath );
+  child->exec->SetEnginePath( EnginePath );
+  child->exec->SetSystemsPath( SystemsPath );
+  child->exec->LoadModel(childAircraft);
+
+  Element* location = el->FindElement("location");
+  if (location) {
+    child->Loc = location->FindElementTripletConvertTo("IN");
+  } else {
+    cerr << endl << highint << fgred << "  No location was found for this child object!" << reset << endl;
+    exit(-1);
   }
-*/
-  if (debug_lvl > 0)  {
-    cout << "      X = " << SlaveFDMList.back()->x << endl;
-    cout << "      Y = " << SlaveFDMList.back()->y << endl;
-    cout << "      Z = " << SlaveFDMList.back()->z << endl;
-    cout << "      Pitch = " << SlaveFDMList.back()->pitch << endl;
-    cout << "      Yaw = " << SlaveFDMList.back()->yaw << endl;
-    cout << "      Roll = " << SlaveFDMList.back()->roll << endl;
+  
+  Element* orientation = el->FindElement("orient");
+  if (orientation) {
+    child->Orient = orientation->FindElementTripletConvertTo("RAD");
+  } else if (debug_lvl > 0) {
+    cerr << endl << highint << "  No orientation was found for this child object! Assuming 0,0,0." << reset << endl;
   }
 
+  ChildFDMList.push_back(child);
+
   return true;
 }
 
@@ -785,6 +899,10 @@ bool FGFDMExec::SetOutputDirectives(string fname)
   result = Output->Load(0);
   Outputs.push_back(Output);
 
+  typedef int (FGOutput::*iOPMF)(void) const;
+  string outputProp = CreateIndexedPropertyName("simulation/output",Outputs.size()-1);
+  instance->Tie(outputProp+"/log_rate_hz", Output, (iOPMF)0, &FGOutput::SetRate);
+
   return result;
 }
 
