@@ -4,7 +4,7 @@
  Date started: 11/17/98
  file The header file for the JSBSim executive.
 
- ------------- Copyright (C) 1999  Jon S. Berndt (jsb@hal-pc.org) -------------
+ ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free Software
@@ -41,16 +41,17 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <models/FGModel.h>
-#include <models/FGOutput.h>
-#include <models/FGInput.h>
-#include <initialization/FGTrim.h>
-#include <initialization/FGInitialCondition.h>
-#include <FGJSBBase.h>
-#include <input_output/FGPropertyManager.h>
-#include <input_output/FGGroundCallback.h>
-#include <input_output/FGXMLFileRead.h>
-#include <models/FGPropagate.h>
+#include "models/FGModel.h"
+#include "models/FGOutput.h"
+#include "models/FGInput.h"
+#include "initialization/FGTrim.h"
+#include "initialization/FGInitialCondition.h"
+#include "FGJSBBase.h"
+#include "input_output/FGPropertyManager.h"
+#include "input_output/FGGroundCallback.h"
+#include "input_output/FGXMLFileRead.h"
+#include "models/FGPropagate.h"
+#include "math/FGColumnVector3.h"
 
 #include <vector>
 #include <string>
@@ -177,6 +178,32 @@ CLASS DECLARATION
 
 class FGFDMExec : public FGJSBBase, public FGXMLFileRead
 {
+  struct childData {
+    FGFDMExec* exec;
+    string info;
+    FGColumnVector3 Loc;
+    FGColumnVector3 Orient;
+    bool mated;
+    bool internal;
+
+    childData(void) {
+      info = "";
+      Loc = FGColumnVector3(0,0,0);
+      Orient = FGColumnVector3(0,0,0);
+      mated = true;
+      internal = false;
+    }
+    
+    void Run(void) {exec->Run();}
+    void AssignState(FGPropagate* source_prop) {
+      exec->GetPropagate()->SetVState(source_prop->GetVState());
+    }
+
+    ~childData(void) {
+      delete exec;
+    }
+  };
+
 public:
 
   /// Default constructor
@@ -195,7 +222,7 @@ public:
       @param model A pointer to the model being scheduled.
       @param rate The rate at which to execute the model as described above.
       @return Currently returns 0 always. */
-  int  Schedule(FGModel* model, int rate);
+  void Schedule(FGModel* model, int rate);
 
   /** This function executes each scheduled model in succession.
       @return true if successful, false if sim should be ended  */
@@ -335,8 +362,12 @@ public:
   FGPropertyManager* GetPropertyManager(void);
   /// Returns a vector of strings representing the names of all loaded models (future)
   vector <string> EnumerateFDMs(void);
-  /// Marks this instance of the Exec object as a "slave" object.
-  void SetSlave(bool s) {IsSlave = s;}
+  /// Gets the number of child FDMs.
+  int GetFDMCount(void) {return ChildFDMList.size();}
+  /// Gets a particular child FDM.
+  childData* GetChildFDM(int i) {return ChildFDMList[i];}
+  /// Marks this instance of the Exec object as a "child" object.
+  void SetChild(bool ch) {IsChild = ch;}
 
   /** Sets the output (logging) mechanism for this run.
       Calling this function passes the name of an output directives file to
@@ -443,7 +474,7 @@ private:
   bool holding;
   bool Constructing;
   bool modelLoaded;
-  bool IsSlave;
+  bool IsChild;
   string modelName;
   string AircraftPath;
   string FullAircraftPath;
@@ -455,29 +486,8 @@ private:
   bool trim_status;
   int ta_mode;
 
-
-  struct slaveData {
-    FGFDMExec* exec;
-    string info;
-    double x, y, z;
-    double roll, pitch, yaw;
-    bool mated;
-
-    slaveData(void) {
-      info = "";
-      x = y = z = 0.0;
-      roll = pitch = yaw = 0.0;
-      mated = true;
-    }
-
-    ~slaveData(void) {
-      delete exec;
-    }
-  };
-
   static FGPropertyManager *master;
 
-  FGModel*            FirstModel;
   FGGroundCallback*   GroundCallback;
   FGState*            State;
   FGAtmosphere*       Atmosphere;
@@ -502,11 +512,13 @@ private:
 
   vector <string> PropertyCatalog;
   vector <FGOutput*> Outputs;
-  vector <slaveData*> SlaveFDMList;
+  vector <childData*> ChildFDMList;
+  vector <FGModel*> Models;
 
   bool ReadFileHeader(Element*);
-  bool ReadSlave(Element*);
+  bool ReadChild(Element*);
   bool ReadPrologue(Element*);
+  void ResetToInitialConditions(int mode);
   bool Allocate(void);
   bool DeAllocate(void);
 
