@@ -82,10 +82,10 @@ FMAX (double a, double b)
 
 class FGFSGroundCallback : public FGGroundCallback {
 public:
-  FGFSGroundCallback(FGInterface* ifc) : mInterface(ifc) {}
+  FGFSGroundCallback(FGJSBsim* ifc) : mInterface(ifc) {}
   virtual ~FGFSGroundCallback() {}
 
-  /** Get the altitude above sea level depenent on the location. */
+  /** Get the altitude above sea level dependent on the location. */
   virtual double GetAltitude(const FGLocation& l) const {
     double pt[3] = { SG_FEET_TO_METER*l(eX),
                      SG_FEET_TO_METER*l(eY),
@@ -100,17 +100,16 @@ public:
                             FGLocation& cont,
                             FGColumnVector3& n, FGColumnVector3& v) const {
     double loc_cart[3] = { l(eX), l(eY), l(eZ) };
-    double contact[3], normal[3], vel[3], lc, ff, agl;
-    int groundtype;
-    mInterface->get_agl_ft(t, loc_cart, contact, normal, vel,
-                           &groundtype, &lc, &ff, &agl);
-    n = l.GetTec2l()*FGColumnVector3( normal[0], normal[1], normal[2] );
-    v = l.GetTec2l()*FGColumnVector3( vel[0], vel[1], vel[2] );
+    double contact[3], normal[3], vel[3], agl = 0;
+    mInterface->get_agl_ft(t, loc_cart, SG_METER_TO_FEET*2, contact, normal,
+                           vel, &agl);
+    n = FGColumnVector3( -normal[0], -normal[1], -normal[2] );
+    v = FGColumnVector3( vel[0], vel[1], vel[2] );
     cont = FGColumnVector3( contact[0], contact[1], contact[2] );
     return agl;
   }
 private:
-  FGInterface* mInterface;
+  FGJSBsim* mInterface;
 };
 
 /******************************************************************************/
@@ -211,7 +210,7 @@ FGJSBsim::FGJSBsim( double dt )
       if (node->getChild("level-gal_us", 0, false) != 0) {
         Propulsion->GetTank(i)->SetContents(node->getDoubleValue("level-gal_us") * 6.6);
       } else {
-        node->setDoubleValue("level-lb", Propulsion->GetTank(i)->GetContents());
+        node->setDoubleValue("level-lbs", Propulsion->GetTank(i)->GetContents());
         node->setDoubleValue("level-gal_us", Propulsion->GetTank(i)->GetContents() / 6.6);
       }
       node->setDoubleValue("capacity-gal_us",
@@ -259,6 +258,10 @@ FGJSBsim::FGJSBsim( double dt )
     speedbrake_pos_pct->setDoubleValue(0);
     spoilers_pos_pct->setDoubleValue(0);
 
+    ab_brake_engaged = fgGetNode("/autopilot/autobrake/engaged", true);
+    ab_brake_left_pct = fgGetNode("/autopilot/autobrake/brake-left-output", true);
+    ab_brake_right_pct = fgGetNode("/autopilot/autobrake/brake-right-output", true);
+    
     temperature = fgGetNode("/environment/temperature-degc",true);
     pressure = fgGetNode("/environment/pressure-inhg",true);
     density = fgGetNode("/environment/density-slugft3",true);
@@ -319,9 +322,9 @@ void FGJSBsim::init()
       Atmosphere->UseInternal();
     }
 
-    fgic->SetVNorthFpsIC( wind_from_north->getDoubleValue() );
-    fgic->SetVEastFpsIC( wind_from_east->getDoubleValue() );
-    fgic->SetVDownFpsIC( wind_from_down->getDoubleValue() );
+    fgic->SetVNorthFpsIC( -wind_from_north->getDoubleValue() );
+    fgic->SetVEastFpsIC( -wind_from_east->getDoubleValue() );
+    fgic->SetVDownFpsIC( -wind_from_down->getDoubleValue() );
 
     //Atmosphere->SetExTemperature(get_Static_temperature());
     //Atmosphere->SetExPressure(get_Static_pressure());
@@ -331,7 +334,7 @@ void FGJSBsim::init()
      << ", " << fdmex->GetAtmosphere()->GetDensity() );
 
     if (fgGetBool("/sim/presets/running")) {
-          for (int i=0; i < Propulsion->GetNumEngines(); i++) {
+          for (unsigned int i=0; i < Propulsion->GetNumEngines(); i++) {
             SGPropertyNode * node = fgGetNode("engines/engine", i, true);
             node->setBoolValue("running", true);
             Propulsion->GetEngine(i)->SetRunning(true);
@@ -385,7 +388,7 @@ void FGJSBsim::init()
     SG_LOG( SG_FLIGHT, SG_INFO, "  Longitude: "
             << Propagate->GetLocation().GetLongitudeDeg() << " deg" );
     SG_LOG( SG_FLIGHT, SG_INFO, "  Altitude: "
-            << Propagate->Geth() << " feet" );
+            << Propagate->GetAltitudeASL() << " feet" );
     SG_LOG( SG_FLIGHT, SG_INFO, "  loaded initial conditions" );
 
     SG_LOG( SG_FLIGHT, SG_INFO, "  set dt" );
@@ -432,26 +435,31 @@ void FGJSBsim::update( double dt )
     double alt, slr, lat, lon;
     FGColumnVector3 cart = Auxiliary->GetLocationVRP();
     if ( needTrim && startup_trim->getBoolValue() ) {
-      alt = fgic->GetAltitudeFtIC();
+      alt = fgic->GetAltitudeASLFtIC();
       slr = fgic->GetSeaLevelRadiusFtIC();
       lat = fgic->GetLatitudeDegIC() * SGD_DEGREES_TO_RADIANS;
       lon = fgic->GetLongitudeDegIC() * SGD_DEGREES_TO_RADIANS;
       cart = FGLocation(lon, lat, alt+slr);
     }
     double cart_pos[3] = { cart(1), cart(2), cart(3) };
-    bool cache_ok = prepare_ground_cache_ft( State->Getsim_time(), cart_pos,
+    double t0 = State->Getsim_time();
+    bool cache_ok = prepare_ground_cache_ft( t0, t0 + dt, cart_pos,
                                              groundCacheRadius );
     if (!cache_ok) {
       SG_LOG(SG_FLIGHT, SG_WARN,
              "FGInterface is being called without scenery below the aircraft!");
-      SG_LOG(SG_FLIGHT, SG_WARN,
-             "altitude         = " << alt);
-      SG_LOG(SG_FLIGHT, SG_WARN,
-            "sea level radius = " << slr);
-      SG_LOG(SG_FLIGHT, SG_WARN,
-            "latitude         = " << lat);
-      SG_LOG(SG_FLIGHT, SG_WARN,
-            "longitude        = " << lon);
+
+      alt = fgic->GetAltitudeASLFtIC();
+      SG_LOG(SG_FLIGHT, SG_WARN, "altitude         = " << alt);
+
+      slr = fgic->GetSeaLevelRadiusFtIC();
+      SG_LOG(SG_FLIGHT, SG_WARN, "sea level radius = " << slr);
+
+      lat = fgic->GetLatitudeDegIC() * SGD_DEGREES_TO_RADIANS;
+      SG_LOG(SG_FLIGHT, SG_WARN, "latitude         = " << lat);
+
+      lon = fgic->GetLongitudeDegIC() * SGD_DEGREES_TO_RADIANS;
+      SG_LOG(SG_FLIGHT, SG_WARN, "longitude        = " << lon);
       //return;
     }
 
@@ -461,18 +469,17 @@ void FGJSBsim::update( double dt )
 
     if ( needTrim ) {
       if ( startup_trim->getBoolValue() ) {
-        double contact[3], dummy[3], lc, ff, agl;
-        int groundtype;
-        get_agl_ft(State->Getsim_time(), cart_pos, contact,
-                   dummy, dummy, &groundtype, &lc, &ff, &agl);
+        double contact[3], d[3], agl;
+        get_agl_ft(State->Getsim_time(), cart_pos, SG_METER_TO_FEET*2, contact,
+                   d, d, &agl);
         double terrain_alt = sqrt(contact[0]*contact[0] + contact[1]*contact[1]
              + contact[2]*contact[2]) - fgic->GetSeaLevelRadiusFtIC();
 
         SG_LOG(SG_FLIGHT, SG_INFO,
-          "Ready to trim, terrain altitude is: "
+          "Ready to trim, terrain elevation is: "
             << terrain_alt * SG_METER_TO_FEET );
 
-        fgic->SetTerrainAltitudeFtIC( terrain_alt );
+        fgic->SetTerrainElevationFtIC( terrain_alt );
         do_trim();
       } else {
         fdmex->RunIC();  //apply any changes made through the set_ functions
@@ -486,8 +493,8 @@ void FGJSBsim::update( double dt )
     }
 
     FGJSBBase::Message* msg;
-    while (fdmex->SomeMessages()) {
-      msg = fdmex->ProcessMessage();
+    while (msg = fdmex->ProcessNextMessage()) {
+//      msg = fdmex->ProcessNextMessage();
       switch (msg->type) {
       case FGJSBBase::Message::eText:
         if (msg->text == "Crash Detected: Simulation FREEZE.")
@@ -539,8 +546,18 @@ bool FGJSBsim::copy_to_JSBsim()
         // Parking brake sets minimum braking
         // level for mains.
     double parking_brake = globals->get_controls()->get_brake_parking();
-    FCS->SetLBrake(FMAX(globals->get_controls()->get_brake_left(), parking_brake));
-    FCS->SetRBrake(FMAX(globals->get_controls()->get_brake_right(), parking_brake));
+    double left_brake = globals->get_controls()->get_brake_left();
+    double right_brake = globals->get_controls()->get_brake_right();
+    
+    if (ab_brake_engaged->getBoolValue()) {
+      left_brake = ab_brake_left_pct->getDoubleValue();
+      right_brake = ab_brake_right_pct->getDoubleValue(); 
+    }
+    
+    FCS->SetLBrake(FMAX(left_brake, parking_brake));
+    FCS->SetRBrake(FMAX(right_brake, parking_brake));
+    
+    
     FCS->SetCBrake( 0.0 );
     // FCS->SetCBrake( globals->get_controls()->get_brake(2) );
 
@@ -586,6 +603,8 @@ bool FGJSBsim::copy_to_JSBsim()
         eng->SetCondition( globals->get_controls()->get_condition(i) );
         break;
         } // end FGTurboProp code block
+      default:
+        break;
       }
 
       { // FGEngine code block
@@ -610,9 +629,9 @@ bool FGJSBsim::copy_to_JSBsim()
     tmp = turbulence_rate->getDoubleValue();
     //Atmosphere->SetTurbRate(tmp);
 
-    Atmosphere->SetWindNED( wind_from_north->getDoubleValue(),
-                            wind_from_east->getDoubleValue(),
-                            wind_from_down->getDoubleValue() );
+    Atmosphere->SetWindNED( -wind_from_north->getDoubleValue(),
+                            -wind_from_east->getDoubleValue(),
+                            -wind_from_down->getDoubleValue() );
 //    SG_LOG(SG_FLIGHT,SG_INFO, "Wind NED: "
 //                  << get_V_north_airmass() << ", "
 //                  << get_V_east_airmass()  << ", "
@@ -622,11 +641,11 @@ bool FGJSBsim::copy_to_JSBsim()
       SGPropertyNode * node = fgGetNode("/consumables/fuel/tank", i, true);
       FGTank * tank = Propulsion->GetTank(i);
       tank->SetContents(node->getDoubleValue("level-gal_us") * 6.6);
-//       tank->SetContents(node->getDoubleValue("level-lb"));
+//       tank->SetContents(node->getDoubleValue("level-lbs"));
     }
 
     Propulsion->SetFuelFreeze((fgGetNode("/sim/freeze/fuel",true))->getBoolValue());
-    fdmex->SetSlave(slaved->getBoolValue());
+    fdmex->SetChild(slaved->getBoolValue());
 
     return true;
 }
@@ -705,9 +724,8 @@ bool FGJSBsim::copy_from_JSBsim()
     {
       double loc_cart[3] = { l(FGJSBBase::eX), l(FGJSBBase::eY), l(FGJSBBase::eZ) };
       double contact[3], d[3], sd, t;
-      int id;
       is_valid_m(&t, d, &sd);
-      get_agl_ft(t, loc_cart, contact, d, d, &id, &sd, &sd, &sd);
+      get_agl_ft(t, loc_cart, SG_METER_TO_FEET*2, contact, d, d, &sd);
       double rwrad
         = FGColumnVector3( contact[0], contact[1], contact[2] ).Magnitude();
       _set_Runway_altitude( rwrad - get_Sea_level_radius() );
@@ -737,9 +755,7 @@ bool FGJSBsim::copy_from_JSBsim()
     // Copy the engine values from JSBSim.
     for ( i=0; i < Propulsion->GetNumEngines(); i++ ) {
       SGPropertyNode * node = fgGetNode("engines/engine", i, true);
-      char buf[30];
-      sprintf(buf, "engines/engine[%d]/thruster", i);
-      SGPropertyNode * tnode = fgGetNode(buf, true);
+      SGPropertyNode * tnode = node->getChild("thruster", 0, true);
       FGThruster * thruster = Propulsion->GetEngine(i)->GetThruster();
 
       switch (Propulsion->GetEngine(i)->GetType()) {
@@ -862,7 +878,7 @@ bool FGJSBsim::copy_from_JSBsim()
         double contents = tank->GetContents();
         double temp = tank->GetTemperature_degC();
         node->setDoubleValue("level-gal_us", contents/6.6);
-        node->setDoubleValue("level-lb", contents);
+        node->setDoubleValue("level-lbs", contents);
         if (temp != -9999.0) node->setDoubleValue("temperature_degC", temp);
       }
     }
@@ -952,6 +968,7 @@ void FGJSBsim::set_Longitude(double lon)
     needTrim=true;
 }
 
+// Sets the altitude above sea level.
 void FGJSBsim::set_Altitude(double alt)
 {
     static SGConstPropertyNode_ptr latitude = fgGetNode("/position/latitude-deg");
@@ -970,9 +987,9 @@ void FGJSBsim::set_Altitude(double alt)
     _set_Sea_level_radius( sea_level_radius_meters * SG_METER_TO_FEET  );
     fgic->SetSeaLevelRadiusFtIC( sea_level_radius_meters * SG_METER_TO_FEET );
     SG_LOG(SG_FLIGHT, SG_INFO,
-          "Terrain altitude: " << cur_fdm_state->get_Runway_altitude() * SG_METER_TO_FEET );
+          "Terrain elevation: " << cur_fdm_state->get_Runway_altitude() * SG_METER_TO_FEET );
     fgic->SetLatitudeRadIC( lat_geoc );
-    fgic->SetAltitudeFtIC(alt);
+    fgic->SetAltitudeASLFtIC(alt);
     needTrim=true;
 }
 
@@ -1086,6 +1103,7 @@ void FGJSBsim::init_gear(void )
       node->setDoubleValue("yoffset-in", gear->GetBodyLocation()(2));
       node->setDoubleValue("zoffset-in", gear->GetBodyLocation()(3));
       node->setBoolValue("wow", gear->GetWOW());
+      node->setDoubleValue("rollspeed-ms", gear->GetWheelRollVel()*0.3043);
       node->setBoolValue("has-brake", gear->GetBrakeGroup() > 0);
       node->setDoubleValue("position-norm", gear->GetGearUnitPos());
       node->setDoubleValue("tire-pressure-norm", gear->GetTirePressure());
@@ -1104,6 +1122,7 @@ void FGJSBsim::update_gear(void)
       FGLGear *gear = gr->GetGearUnit(i);
       SGPropertyNode * node = fgGetNode("gear/gear", i, true);
       node->getChild("wow", 0, true)->setBoolValue( gear->GetWOW());
+      node->getChild("rollspeed-ms", 0, true)->setDoubleValue(gear->GetWheelRollVel()*0.3043);
       node->getChild("position-norm", 0, true)->setDoubleValue(gear->GetGearUnitPos());
       gear->SetTirePressure(node->getDoubleValue("tire-pressure-norm"));
       node->setDoubleValue("compression-norm", gear->GetCompLen());
@@ -1156,7 +1175,7 @@ void FGJSBsim::update_ic(void)
    if ( !needTrim ) {
      fgic->SetLatitudeRadIC(get_Lat_geocentric() );
      fgic->SetLongitudeRadIC( get_Longitude() );
-     fgic->SetAltitudeFtIC( get_Altitude() );
+     fgic->SetAltitudeASLFtIC( get_Altitude() );
      fgic->SetVcalibratedKtsIC( get_V_calibrated_kts() );
      fgic->SetThetaRadIC( get_Theta() );
      fgic->SetPhiRadIC( get_Phi() );
@@ -1165,6 +1184,23 @@ void FGJSBsim::update_ic(void)
    }
 }
 
+bool
+FGJSBsim::get_agl_ft(double t, const double pt[3], double alt_off,
+                     double contact[3], double normal[3], double vel[3],
+                     double *agl)
+{
+   double angularVel[3];
+   const SGMaterial* material;
+   simgear::BVHNode::Id id;
+   if (!FGInterface::get_agl_ft(t, pt, alt_off, contact, normal, vel,
+                                angularVel, material, id))
+       return false;
+   SGGeod geodPt = SGGeod::fromCart(SG_FEET_TO_METER*SGVec3d(pt));
+   SGQuatd hlToEc = SGQuatd::fromLonLat(geodPt);
+   *agl = dot(hlToEc.rotate(SGVec3d(0, 0, 1)), SGVec3d(contact) - SGVec3d(pt));
+   return true;
+}
+
 inline static double dot3(const FGColumnVector3& a, const FGColumnVector3& b)
 {
     return a(1) * b(1) + a(2) * b(2) + a(3) * b(3);
@@ -1237,12 +1273,10 @@ void FGJSBsim::update_external_forces(double t_off)
     double contact[3];
     double ground_normal[3];
     double ground_vel[3];
-    int ground_type;
-    const SGMaterial* ground_material;
     double root_agl_ft;
 
     if (!got_wire) {
-        bool got = get_agl_ft(t_off, hook_area[1], 0, contact, ground_normal, ground_vel, &ground_type, &ground_material, &root_agl_ft);
+        bool got = get_agl_ft(t_off, hook_area[1], 0, contact, ground_normal, ground_vel, &root_agl_ft);
         if (got && root_agl_ft > 0 && root_agl_ft < hook_length) {
             FGColumnVector3 ground_normal_body = Tl2b * (Tec2l * FGColumnVector3(ground_normal[0], ground_normal[1], ground_normal[2]));
             FGColumnVector3 contact_body = Tl2b * Location.LocationToLocal(FGColumnVector3(contact[0], contact[1], contact[2]));
