@@ -4,7 +4,7 @@
  Author:       Jon S. Berndt
  Date started: 1/5/99
 
- ------------- Copyright (C) 1999  Jon S. Berndt (jsb@hal-pc.org) -------------
+ ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free Software
@@ -38,12 +38,11 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <models/FGModel.h>
-#include <math/FGColumnVector3.h>
-#include <initialization/FGInitialCondition.h>
-#include <math/FGLocation.h>
-#include <math/FGQuaternion.h>
-#include <math/FGMatrix33.h>
+#include "models/FGModel.h"
+#include "math/FGColumnVector3.h"
+#include "math/FGLocation.h"
+#include "math/FGQuaternion.h"
+#include "math/FGMatrix33.h"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 DEFINITIONS
@@ -57,6 +56,8 @@ FORWARD DECLARATIONS
 
 namespace JSBSim {
 
+class FGInitialCondition;
+
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -97,6 +98,27 @@ CLASS DECLARATION
 
 class FGPropagate : public FGModel {
 public:
+
+  /** The current vehicle state vector structure contains the translational and
+    angular position, and the translational and angular velocity. */
+  struct VehicleState {
+    /** Represents the current location of the vehicle in Earth centered Earth
+        fixed (ECEF) frame.
+        units ft */
+    FGLocation vLocation;
+    /** The velocity vector of the vehicle with respect to the ECEF frame,
+        expressed in the body system.
+        units ft/sec */
+    FGColumnVector3 vUVW;
+    /** The angular velocity vector for the vehicle relative to the ECEF frame,
+        expressed in the body frame.
+        units rad/sec */
+    FGColumnVector3 vPQR;
+    /** The current orientation of the vehicle, that is, the orientation of the
+        body frame relative to the local, vehilce-carried, NED frame. */
+    FGQuaternion vQtrn;
+  };
+
   /** Constructor.
       The constructor initializes several variables, and sets the initial set
       of integrators to use as follows:
@@ -113,26 +135,6 @@ public:
   /// These define the indices use to select the various integrators.
   enum eIntegrateType {eNone = 0, eRectEuler, eTrapezoidal, eAdamsBashforth2, eAdamsBashforth3};
 
-/** The current vehicle state vector structure contains the translational and
-    angular position, and the translational and angular velocity. */
-struct VehicleState {
-  /** Represents the current location of the vehicle in Earth centered Earth
-      fixed (ECEF) frame.
-      units ft */
-  FGLocation vLocation;
-  /** The velocity vector of the vehicle with respect to the ECEF frame,
-      expressed in the body system.
-      units ft/sec */
-  FGColumnVector3 vUVW;
-  /** The angular velocity vector for the vehicle relative to the ECEF frame,
-      expressed in the body frame.
-      units rad/sec */
-  FGColumnVector3 vPQR;
-  /** The current orientation of the vehicle, that is, the orientation of the
-      body frame relative to the local, vehilce-carried, NED frame. */
-  FGQuaternion vQtrn;
-};
-
   /** Initializes the FGPropagate class after instantiation and prior to first execution.
       The base class FGModel::InitModel is called first, initializing pointers to the 
       other FGModel objects (and others).  */
@@ -181,7 +183,7 @@ struct VehicleState {
   */
   const FGColumnVector3& GetUVWdot(void) const { return vUVWdot; }
   
-  /** Retrieves the body angular rates vector.
+  /** Retrieves the body angular rates vector, relative to the ECEF frame.
       Retrieves the body angular rates (p, q, r), which are calculated by integration
       of the angular acceleration.
       The vector returned is represented by an FGColumnVector reference. The vector
@@ -195,6 +197,20 @@ struct VehicleState {
   */
   const FGColumnVector3& GetPQR(void) const {return VState.vPQR;}
   
+  /** Retrieves the body angular rates vector, relative to the ECI (inertial) frame.
+      Retrieves the body angular rates (p, q, r), which are calculated by integration
+      of the angular acceleration.
+      The vector returned is represented by an FGColumnVector reference. The vector
+      for the angular velocity in Body frame is organized (P, Q, R). The vector
+      is 1-based, so that the first element can be retrieved using the "()" operator.
+      In other words, vPQR(1) is P. Various convenience enumerators are defined
+      in FGJSBBase. The relevant enumerators for the vector returned by this call are,
+      eP=1, eQ=2, eR=3.
+      units rad/sec
+      @return The body frame angular rates in rad/sec.
+  */
+  const FGColumnVector3& GetPQRi(void) const {return vPQRi;}
+
   /** Retrieves the body axis angular acceleration vector.
       Retrieves the body axis angular acceleration vector in rad/sec^2. The
       angular acceleration vector is determined from the applied forces and
@@ -270,23 +286,21 @@ struct VehicleState {
   */
   double GetInertialVelocityMagnitude(void) const { return vInertialVelocity.Magnitude(); }
 
-  /** Returns the current altitude.
-      Returns the current altitude. Specifically, this function returns the
-      difference between the distance to the center of the Earth, and sea level.
+  /** Returns the current altitude above sea level.
+      This function returns the altitude above sea level.
       units ft
       @return The current altitude above sea level in feet.
   */
-  double Geth(void)   const { return VState.vLocation.GetRadius() - SeaLevelRadius; }
+  double GetAltitudeASL(void)   const { return VState.vLocation.GetRadius() - SeaLevelRadius; }
 
-  /** Returns the current altitude.
-      Returns the curren altitude. Specifically, this function returns the
-      difference between the distance to the center of the Earth, and sea level.
+  /** Returns the current altitude above sea level.
+      This function returns the altitude above sea level.
       units meters
       @return The current altitude above sea level in meters.
   */
-  double Gethmeters(void) const { return Geth()*fttom;}
+  double GetAltitudeASLmeters(void) const { return GetAltitudeASL()*fttom;}
 
-  /** Retrieves a body frame angular velocity component.
+  /** Retrieves a body frame angular velocity component relative to the ECEF frame.
       Retrieves a body frame angular velocity component. The angular velocity
       returned is extracted from the vPQR vector (an FGColumnVector). The vector
       for the angular velocity in Body frame is organized (P, Q, R). The vector
@@ -299,6 +313,19 @@ struct VehicleState {
   */
   double GetPQR(int axis) const {return VState.vPQR(axis);}
 
+  /** Retrieves a body frame angular velocity component relative to the ECI (inertial) frame.
+      Retrieves a body frame angular velocity component. The angular velocity
+      returned is extracted from the vPQR vector (an FGColumnVector). The vector
+      for the angular velocity in Body frame is organized (P, Q, R). The vector
+      is 1-based. In other words, GetPQR(1) returns P (roll rate). Various
+      convenience enumerators are defined in FGJSBBase. The relevant enumerators
+      for the angular velocity returned by this call are, eP=1, eQ=2, eR=3.
+      units rad/sec
+      @param axis the index of the angular velocity component desired (1-based).
+      @return The body frame angular velocity component.
+  */
+  double GetPQRi(int axis) const {return vPQRi(axis);}
+
   /** Retrieves a body frame angular acceleration component.
       Retrieves a body frame angular acceleration component. The angular
       acceleration returned is extracted from the vPQRdot vector (an
@@ -356,16 +383,16 @@ struct VehicleState {
   */
   double Gethdot(void) const { return -vVel(eDown); }
 
-  /** Returns the "constant" RunwayRadius.
-      The RunwayRadius parameter is set by the calling application or set to
-      sea level if JSBSim is running in standalone mode.
+  /** Returns the "constant" LocalTerrainRadius.
+      The LocalTerrainRadius parameter is set by the calling application or set to
+      sea level + terrain elevation if JSBSim is running in standalone mode.
       units feet
-      @return distance of the runway from the center of the earth.
+      @return distance of the local terrain from the center of the earth.
       */
-  double GetRunwayRadius(void) const;
+  double GetLocalTerrainRadius(void) const;
 
   double GetSeaLevelRadius(void) const { return SeaLevelRadius; }
-  double GetTerrainElevationASL(void) const;
+  double GetTerrainElevation(void) const;
   double GetDistanceAGL(void)  const;
   double GetRadius(void) const {
       if (VState.vLocation.GetRadius() == 0) return 1.0;
@@ -423,13 +450,13 @@ struct VehicleState {
       @return a reference to the local-to-ECEF matrix.  */
   const FGMatrix33& GetTl2ec(void) const { return VState.vLocation.GetTl2ec(); }
 
-  const VehicleState GetVState(void) const { return VState; }
+  VehicleState* GetVState(void) { return &VState; }
 
-  void SetVState(VehicleState vstate) {
-      VState.vLocation = vstate.vLocation;
-      VState.vUVW = vstate.vUVW;
-      VState.vPQR = vstate.vPQR;
-      VState.vQtrn = vstate.vQtrn; // ... mmh
+  void SetVState(VehicleState* vstate) {
+      VState.vLocation = vstate->vLocation;
+      VState.vUVW = vstate->vUVW;
+      VState.vPQR = vstate->vPQR;
+      VState.vQtrn = vstate->vQtrn; // ... mmh
   }
 
   const FGQuaternion GetQuaternion(void) const { return VState.vQtrn; }
@@ -452,13 +479,18 @@ struct VehicleState {
   void SetLatitudeDeg(double lat) {SetLatitude(lat*degtorad);}
   void SetRadius(double r) { VState.vLocation.SetRadius(r); }
   void SetLocation(const FGLocation& l) { VState.vLocation = l; }
-  void Seth(double tt);
-  void Sethmeters(double tt) {Seth(tt/fttom);}
+  void SetAltitudeASL(double altASL);
+  void SetAltitudeASLmeters(double altASL) {SetAltitudeASL(altASL/fttom);}
   void SetSeaLevelRadius(double tt) { SeaLevelRadius = tt; }
-  void SetTerrainElevationASL(double tt);
+  void SetTerrainElevation(double tt);
   void SetDistanceAGL(double tt);
   void SetInitialState(const FGInitialCondition *);
-  void RecomputeRunwayRadius(void);
+  void RecomputeLocalTerrainRadius(void);
+
+  void NudgeBodyLocation(FGColumnVector3 deltaLoc) {
+    vDeltaXYZEC = GetTb2ec()*deltaLoc;
+    VState.vLocation -= vDeltaXYZEC;
+  }
 
   void CalculatePQRdot(void);
   void CalculateQuatdot(void);
@@ -477,6 +509,7 @@ private:
   FGColumnVector3 vUVWdot, last_vUVWdot, last2_vUVWdot;
   FGColumnVector3 vLocationDot, last_vLocationDot, last2_vLocationDot;
   FGColumnVector3 vLocation;
+  FGColumnVector3 vDeltaXYZEC;
   FGColumnVector3 vPQRi;   // Inertial frame angular velocity
   FGColumnVector3 vOmega;  // The Earth angular velocity vector
   FGColumnVector3 vOmegaLocal;  // The local frame angular velocity vector
@@ -492,7 +525,7 @@ private:
   FGMatrix33 Ti2b;   // ECI to body frame rotation matrix
   FGMatrix33 Tb2i;   // body to ECI frame rotation matrix
   
-  double RunwayRadius, SeaLevelRadius, VehicleRadius;
+  double LocalTerrainRadius, SeaLevelRadius, VehicleRadius;
   double radInv;
   int integrator_rotational_rate;
   int integrator_translational_rate;
@@ -504,4 +537,7 @@ private:
 };
 }
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+#include "initialization/FGInitialCondition.h"
+
 #endif
