@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Id: gui.cxx,v 1.22.2.1 2006-11-01 22:54:32 mfranz Exp $
+ * $Id$
  **************************************************************************/
 
 
@@ -30,9 +30,7 @@
 
 #include <simgear/compiler.h>
 
-#ifdef HAVE_WINDOWS_H
-#  include <windows.h>
-#endif
+#include <string>
 
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
@@ -45,12 +43,15 @@
 #include <Main/main.hxx>
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/WindowSystemAdapter.hxx>
 #include <GUI/new_gui.hxx>
 
 #include "gui.h"
-#include "gui_local.hxx"
 #include "layout.hxx"
 
+#include <osg/GraphicsContext>
+
+using namespace flightgear;
 
 puFont guiFnt = 0;
 
@@ -59,48 +60,61 @@ puFont guiFnt = 0;
 init the gui
 _____________________________________________________________________*/
 
-
-void guiInit()
+namespace
 {
-    char *mesa_win_state;
-
-    // Initialize PUI
-#ifndef PU_USE_NONE
-    puInit();
-#endif
-    puSetDefaultStyle         ( PUSTYLE_SMALL_SHADED ); //PUSTYLE_DEFAULT
-    puSetDefaultColourScheme  (0.8, 0.8, 0.9, 1);
-
-    FGFontCache *fc = globals->get_fontcache();
-    puFont *GuiFont = fc->get(globals->get_locale()->getStringValue("font", "typewriter.txf"), 15);
-    puSetDefaultFonts(*GuiFont, *GuiFont);
-    guiFnt = puGetDefaultLabelFont();
-
-    LayoutWidget::setDefaultFont(GuiFont, 15);
-  
-    if (!fgHasNode("/sim/startup/mouse-pointer")) {
-        // no preference specified for mouse pointer, attempt to autodetect...
-        // Determine if we need to render the cursor, or if the windowing
-        // system will do it.  First test if we are rendering with glide.
-        if ( strstr ( general.get_glRenderer(), "Glide" ) ) {
-            // Test for the MESA_GLX_FX env variable
-            if ( (mesa_win_state = getenv( "MESA_GLX_FX" )) != NULL) {
-                // test if we are fullscreen mesa/glide
-                if ( (mesa_win_state[0] == 'f') ||
-                     (mesa_win_state[0] == 'F') ) {
-                    puShowCursor ();
-                }
-            }
-        }
-//        mouse_active = ~mouse_active;
-    } else if ( !fgGetBool("/sim/startup/mouse-pointer") ) {
-        // don't show pointer
-    } else {
-        // force showing pointer
-        puShowCursor();
-//        mouse_active = ~mouse_active;
+class GUIInitOperation : public GraphicsContextOperation
+{
+public:
+    GUIInitOperation() : GraphicsContextOperation(std::string("GUI init"))
+    {
     }
-	
-    // MOUSE_VIEW mode stuff
-	initMouseQuat();
+    void run(osg::GraphicsContext* gc)
+    {
+        WindowSystemAdapter* wsa = WindowSystemAdapter::getWSA();
+        wsa->puInitialize();
+        puSetDefaultStyle         ( PUSTYLE_SMALL_SHADED ); //PUSTYLE_DEFAULT
+        puSetDefaultColourScheme  (0.8, 0.8, 0.9, 1);
+
+        FGFontCache *fc = globals->get_fontcache();
+        fc->initializeFonts();
+        puFont *GuiFont
+            = fc->get(globals->get_locale()->getStringValue("font",
+                                                            "typewriter.txf"),
+                      15);
+        puSetDefaultFonts(*GuiFont, *GuiFont);
+        guiFnt = puGetDefaultLabelFont();
+
+        LayoutWidget::setDefaultFont(GuiFont, 15);
+  
+        if (!fgHasNode("/sim/startup/mouse-pointer")) {
+            // no preference specified for mouse pointer
+        } else if ( !fgGetBool("/sim/startup/mouse-pointer") ) {
+            // don't show pointer
+        } else {
+            // force showing pointer
+            puShowCursor();
+        }
+    }
+};
+
+osg::ref_ptr<GUIInitOperation> initOp;
 }
+
+void guiStartInit(osg::GraphicsContext* gc)
+{
+    if (gc) {
+        initOp = new GUIInitOperation;
+        gc->add(initOp.get());
+    }
+}
+
+bool guiFinishInit()
+{
+    if (!initOp.valid())
+        return true;
+    if (!initOp->isFinished())
+        return false;
+    initOp = 0;
+    return true;
+}
+
