@@ -61,7 +61,7 @@ extern void fgDumpSnapShotWrapper();
 extern void fgHiResDumpWrapper();
 extern void fgHiResDump();
 #endif
-#if defined( WIN32 ) && !defined( __CYGWIN__) && !defined(__MINGW32__)
+#if defined( _WIN32 ) && !defined(__MINGW32__)
 extern void printScreen();
 #endif
 extern void helpCb();
