@@ -4,6 +4,7 @@
 
 #include <plib/pu.h>
 #include <simgear/props/props.hxx>
+#include <simgear/math/SGMath.hxx>
 
 #include "layout.hxx"
 
@@ -82,9 +83,9 @@ int LayoutWidget::getNum(const char* f)
     return _prop->getIntValue(f);
 }
 
-bool LayoutWidget::getBool(const char* f)
+bool LayoutWidget::getBool(const char* f, bool dflt)
 {
-    return _prop->getBoolValue(f);
+    return _prop->getBoolValue(f, dflt);
 }
 
 const char* LayoutWidget::getStr(const char* f)
