@@ -44,25 +44,30 @@ typedef string stdString;      // puObject has a "string" member
 
 static string getValueTypeString(const SGPropertyNode *node)
 {
+    using namespace simgear;
     string result;
 
-    SGPropertyNode::Type type = node->getType();
-    if (type == SGPropertyNode::UNSPECIFIED)
+    props::Type type = node->getType();
+    if (type == props::UNSPECIFIED)
         result = "unspecified";
-    else if (type == SGPropertyNode::NONE)
+    else if (type == props::NONE)
         result = "none";
-    else if (type == SGPropertyNode::BOOL)
+    else if (type == props::BOOL)
         result = "bool";
-    else if (type == SGPropertyNode::INT)
+    else if (type == props::INT)
         result = "int";
-    else if (type == SGPropertyNode::LONG)
+    else if (type == props::LONG)
         result = "long";
-    else if (type == SGPropertyNode::FLOAT)
+    else if (type == props::FLOAT)
         result = "float";
-    else if (type == SGPropertyNode::DOUBLE)
+    else if (type == props::DOUBLE)
         result = "double";
-    else if (type == SGPropertyNode::STRING)
+    else if (type == props::STRING)
         result = "string";
+    else if (type == props::VEC3D)
+        result = "vec3d";
+    else if (type == props::VEC4D)
+        result = "vec4d";
 
     return result;
 }
@@ -70,11 +75,12 @@ static string getValueTypeString(const SGPropertyNode *node)
 
 static void dumpProperties(const SGPropertyNode *node)
 {
+    using namespace simgear;
     cout << node->getPath() << '/' << endl;
     for (int i = 0; i < node->nChildren(); i++) {
         const SGPropertyNode *c = node->getChild(i);
-        SGPropertyNode::Type type = c->getType();
-        if (type == SGPropertyNode::ALIAS || c->nChildren())
+        props::Type type = c->getType();
+        if (type == props::ALIAS || c->nChildren())
             continue;
 
         int index = c->getIndex();
@@ -84,21 +90,25 @@ static void dumpProperties(const SGPropertyNode *node)
         cout << " = ";
 
         switch (c->getType()) {
-        case SGPropertyNode::DOUBLE:
-        case SGPropertyNode::FLOAT:
-            cout << std::setprecision(15) << c->getDoubleValue();
+        case props::DOUBLE:
+        case props::FLOAT:
+        case props::VEC3D:
+        case props::VEC4D:
+        {
+            streamsize precision = cout.precision(15);
+            c->printOn(cout);
+            cout.precision(precision);
+        }
             break;
-        case SGPropertyNode::LONG:
-        case SGPropertyNode::INT:
-            cout << c->getLongValue();
+        case props::LONG:
+        case props::INT:
+        case props::BOOL:
+            c->printOn(cout);
             break;
-        case SGPropertyNode::BOOL:
-            cout << (c->getBoolValue() ? "true" : "false");
-            break;
-        case SGPropertyNode::STRING:
+        case props::STRING:
             cout << '"' << c->getStringValue() << '"';
             break;
-        case SGPropertyNode::NONE:
+        case props::NONE:
             break;
         default:
             cout << '\'' << c->getStringValue() << '\'';
@@ -195,7 +205,9 @@ void PropertyList::handle_select(puObject *list_box)
 
         if (prop_list->_dot_files && (selected < 2)) {
             if (src[0] == '.' && (src[1] == '\0' || src[1] == ' ')) {
-                if (mod_ctrl)
+                if (mod_ctrl && mod_shift)
+                    prop_list->_curr->fireValueChanged();
+                else if (mod_ctrl)
                     prop_list->toggleVerbosity();
                 else if (mod_shift)
                     dumpProperties(prop_list->_curr);
@@ -230,7 +242,7 @@ void PropertyList::handle_select(puObject *list_box)
         }
 
         // it is a regular property
-        if (child->getType() == SGPropertyNode::BOOL && mod_ctrl) {
+        if (child->getType() == simgear::props::BOOL && mod_ctrl) {
             child->setBoolValue(!child->getBoolValue());
             prop_list->update(true);
         } else
@@ -297,6 +309,7 @@ void PropertyList::update(bool restore_pos)
 
 void PropertyList::updateTextForEntry(NodeData& data)
 {
+    using namespace simgear;
     SGPropertyNode *node = data.node;
     stdString name = node->getDisplayName(true);
     stdString type = getValueTypeString(node);
@@ -310,8 +323,8 @@ void PropertyList::updateTextForEntry(NodeData& data)
         line << '/';
 
     if (!children || (_verbose && node->hasValue())) {
-        if (node->getType() == SGPropertyNode::STRING
-                || node->getType() == SGPropertyNode::UNSPECIFIED)
+        if (node->getType() == props::STRING
+                || node->getType() == props::UNSPECIFIED)
             sanitize(value);
 
         line << " = '" << value << "' (" << type;
