@@ -26,7 +26,7 @@
 #endif
 
 #include "kln89_page_ndb.hxx"
-#include "Navaids/navrecord.hxx"
+#include <Navaids/navrecord.hxx>
 
 KLN89NDBPage::KLN89NDBPage(KLN89* parent) 
 : KLN89Page(parent) {
@@ -90,7 +90,7 @@ void KLN89NDBPage::Update(double dt) {
 		}
 		if(_subPage == 0) {
 			// TODO - trim VOR-DME from the name, convert to uppercase, abbreviate, etc
-			_kln89->DrawText(np->get_name(), 2, 0, 2);
+			_kln89->DrawText(np->name(), 2, 0, 2);
 			_kln89->DrawLatitude(np->get_lat(), 2, 3, 1);
 			_kln89->DrawLongitude(np->get_lon(), 2, 3, 0);
 		} else {
@@ -158,8 +158,12 @@ void KLN89NDBPage::ClrPressed() {
 void KLN89NDBPage::EntPressed() {
 	if(_entInvert) {
 		_entInvert = false;
-		_last_ndb_id = _ndb_id;
-		_ndb_id = _save_ndb_id;
+		if(_kln89->_dtoReview) {
+			_kln89->DtoInitiate(_ndb_id);
+		} else {
+			_last_ndb_id = _ndb_id;
+			_ndb_id = _save_ndb_id;
+		}
 	}
 }
 
