@@ -146,7 +146,8 @@ DME::update (double delta_time_sec)
         }
         _distance_node->setDoubleValue( tmp_dist );
         _speed_node->setDoubleValue(speed_kt);
-        _time_node->setDoubleValue(distance_nm/speed_kt*60.0);
+        if (SGLimitsd::min() < fabs(speed_kt))
+          _time_node->setDoubleValue(distance_nm/speed_kt*60.0);
         
     } else {
         _last_distance_nm = 0;
@@ -165,14 +166,14 @@ DME::search (double frequency_mhz, double longitude_rad,
     _time_before_search_sec = 1.0;
 
     // try the ILS list first
-    FGNavRecord *dme
-        = globals->get_dmelist()->findByFreq( frequency_mhz, longitude_rad,
-                                              latitude_rad, altitude_m);
+    
+    FGNavRecord *dme = globals->get_dmelist()->findByFreq( frequency_mhz,
+      SGGeod::fromRadM(longitude_rad, latitude_rad, altitude_m));
 
     _transmitter_valid = (dme != NULL);
 
     if ( _transmitter_valid ) {
-        _transmitter = dme->get_cart();
+        _transmitter = dme->cart();
         _transmitter_elevation_ft = dme->get_elev_ft();
         _transmitter_range_nm = dme->get_range();
         _transmitter_bias = dme->get_multiuse();
