@@ -29,6 +29,7 @@
 
 #include <simgear/compiler.h>
 #include <simgear/math/sg_random.h>
+#include <simgear/misc/sg_path.hxx>
 
 #include <Aircraft/aircraft.hxx>
 #include <Navaids/navlist.hxx>
@@ -48,7 +49,8 @@ FGMarkerBeacon::FGMarkerBeacon(SGPropertyNode *node) :
     inner_blink(false),
     name("marker-beacon"),
     num(0),
-    _time_before_search_sec(0.0)
+    _time_before_search_sec(0.0),
+    _sgr(NULL)
 {
     SGPath path( globals->get_fg_root() );
     SGPath term = path;
@@ -62,8 +64,7 @@ FGMarkerBeacon::FGMarkerBeacon(SGPropertyNode *node) :
     low_tbl = new SGInterpTable( low.str() );
     high_tbl = new SGInterpTable( high.str() );
 
-    int i;
-    for ( i = 0; i < node->nChildren(); ++i ) {
+    for ( int i = 0; i < node->nChildren(); ++i ) {
         SGPropertyNode *child = node->getChild(i);
         string cname = child->getName();
         string cval = child->getStringValue();
@@ -99,7 +100,7 @@ FGMarkerBeacon::init ()
 
     SGPropertyNode *node = fgGetNode(branch.c_str(), num, true );
     // Inputs
-    sound_pause = fgGetNode("/sim/sound/pause", false);
+    sound_working = fgGetNode("/sim/sound/working", true);
     lon_node = fgGetNode("/position/longitude-deg", true);
     lat_node = fgGetNode("/position/latitude-deg", true);
     alt_node = fgGetNode("/position/altitude-ft", true);
@@ -109,9 +110,16 @@ FGMarkerBeacon::init ()
     audio_vol = node->getChild("volume", 0, true);
     serviceable = node->getChild("serviceable", 0, true);
 
-    power_btn->setBoolValue( true );
-    audio_btn->setBoolValue( true );
-    serviceable->setBoolValue( true );
+    if (power_btn->getType() == simgear::props::NONE)
+        power_btn->setBoolValue( true );
+    if (audio_btn->getType() == simgear::props::NONE)
+        audio_btn->setBoolValue( true );
+    if (serviceable->getType() == simgear::props::NONE)
+        serviceable->setBoolValue( true );
+
+    SGSoundMgr *smgr = globals->get_soundmgr();
+    _sgr = smgr->find("avionics", true);
+    _sgr->tie_to_listener();
 
     morse.init();
     beacon.init();
@@ -164,23 +172,22 @@ FGMarkerBeacon::update(double dt)
 
     _time_before_search_sec -= dt;
     if ( _time_before_search_sec < 0 ) {
-	search();
+        search();
     }
 
     if ( has_power() && serviceable->getBoolValue()
-            && !sound_pause->getBoolValue()) {
+            && sound_working->getBoolValue()) {
 
         // marker beacon blinking
         bool light_on = ( outer_blink || middle_blink || inner_blink );
-        SGTimeStamp current;
-        current.stamp();
+        SGTimeStamp current = SGTimeStamp::now();
 
-        if ( light_on && (current - blink > 400000) ) {
+        if ( light_on && blink + SGTimeStamp::fromUSec(400000) < current ) {
             light_on = false;
-            blink.stamp();
-        } else if ( !light_on && (current - blink > 100000) ) {
+            blink = current;
+        } else if ( !light_on && blink + SGTimeStamp::fromUSec(100000) < current ) {
             light_on = true;
-            blink.stamp();
+            blink = current;
         }
 
         if ( outer_marker ) {
@@ -210,20 +217,20 @@ FGMarkerBeacon::update(double dt)
 
 
 static bool check_beacon_range( const SGGeod& pos,
-                                FGNavRecord *b )
+                                FGPositioned *b )
 {
-    double d = distSqr(b->get_cart(), SGVec3d::fromGeod(pos));
+    double d = distSqr(b->cart(), SGVec3d::fromGeod(pos));
     // cout << "  distance = " << d << " ("
     //      << FG_ILS_DEFAULT_RANGE * SG_NM_TO_METER
     //         * FG_ILS_DEFAULT_RANGE * SG_NM_TO_METER
     //      << ")" << endl;
 
-    // cout << "  range = " << sqrt(d) << endl;
+    //std::cout << "  range = " << sqrt(d) << std::endl;
 
     // cout << "elev = " << elev * SG_METER_TO_FEET
     //      << " current->get_elev() = " << current->get_elev() << endl;
     double elev_ft = pos.getElevationFt();
-    double delev = elev_ft - b->get_elev_ft();
+    double delev = elev_ft - b->elevation();
 
     // max range is the area under r = 2.4 * alt or r^2 = 4000^2 - alt^2
     // whichever is smaller.  The intersection point is 1538 ...
@@ -236,7 +243,7 @@ static bool check_beacon_range( const SGGeod& pos,
         maxrange2 = 0.0;
     }
     maxrange2 *= SG_FEET_TO_METER * SG_FEET_TO_METER; // convert to meter^2
-    // cout << "delev = " << delev << " maxrange = " << maxrange << endl;
+    //std::cout << "delev = " << delev << " maxrange = " << sqrt(maxrange2) << std::endl;
 
     // match up to twice the published range so we can model
     // reduced signal strength
@@ -247,10 +254,22 @@ static bool check_beacon_range( const SGGeod& pos,
     }
 }
 
+class BeaconFilter : public FGPositioned::Filter
+{
+public:
+  virtual FGPositioned::Type minType() const {
+    return FGPositioned::OM;
+  }
+
+  virtual FGPositioned::Type maxType()  const {
+    return FGPositioned::IM;
+  }
+
+};
+
 // Update current nav/adf radio stations based on current postition
 void FGMarkerBeacon::search()
 {
-
     // reset search time
     _time_before_search_sec = 1.0;
 
@@ -264,13 +283,9 @@ void FGMarkerBeacon::search()
     // Beacons.
     ////////////////////////////////////////////////////////////////////////
 
-    // get closest marker beacon
-    FGNavRecord *b
-      = globals->get_mkrlist()->findClosest( pos.getLongitudeRad(),
-                                             pos.getLatitudeRad(),
-                                             pos.getElevationM() );
-
-    // cout << "marker beacon = " << b << " (" << b->get_type() << ")" << endl;
+    // get closest marker beacon - within a 1nm cutoff
+    BeaconFilter filter;
+    FGPositionedRef b = FGPositioned::findClosest(pos, 1.0, &filter);
 
     fgMkrBeacType beacon_type = NOBEACON;
     bool inrange = false;
@@ -282,8 +297,7 @@ void FGMarkerBeacon::search()
         } else if ( b->type() == FGPositioned::IM ) {
             beacon_type = INNER;
         }
-        inrange = check_beacon_range( pos, b );
-        // cout << "  inrange = " << inrange << endl;
+        inrange = check_beacon_range( pos, b.ptr() );
     }
 
     outer_marker = middle_marker = inner_marker = false;
@@ -291,9 +305,9 @@ void FGMarkerBeacon::search()
     if ( b == NULL || !inrange || !has_power() || !serviceable->getBoolValue() )
     {
         // cout << "no marker" << endl;
-        globals->get_soundmgr()->stop( "outer-marker" );
-        globals->get_soundmgr()->stop( "middle-marker" );
-        globals->get_soundmgr()->stop( "inner-marker" );
+        _sgr->stop( "outer-marker" );
+        _sgr->stop( "middle-marker" );
+        _sgr->stop( "inner-marker" );
     } else {
 
         string current_sound_name;
@@ -303,63 +317,63 @@ void FGMarkerBeacon::search()
             current_sound_name = "outer-marker";
             // cout << "OUTER MARKER" << endl;
             if ( last_beacon != OUTER ) {
-                if ( ! globals->get_soundmgr()->exists( current_sound_name ) ) {
+                if ( ! _sgr->exists( current_sound_name ) ) {
                     SGSoundSample *sound = beacon.get_outer();
                     if ( sound ) {
-                        globals->get_soundmgr()->add( sound, current_sound_name );
+                        _sgr->add( sound, current_sound_name );
                     }
                 }
             }
             if ( audio_btn->getBoolValue() ) {
-                if ( !globals->get_soundmgr()->is_playing(current_sound_name) ) {
-                    globals->get_soundmgr()->play_looped( current_sound_name );
+                if ( !_sgr->is_playing(current_sound_name) ) {
+                    _sgr->play_looped( current_sound_name );
                 }
             } else {
-                globals->get_soundmgr()->stop( current_sound_name );
+                _sgr->stop( current_sound_name );
             }
         } else if ( beacon_type == MIDDLE ) {
             middle_marker = true;
             current_sound_name = "middle-marker";
             // cout << "MIDDLE MARKER" << endl;
             if ( last_beacon != MIDDLE ) {
-                if ( ! globals->get_soundmgr()->exists( current_sound_name ) ) {
+                if ( ! _sgr->exists( current_sound_name ) ) {
                     SGSoundSample *sound = beacon.get_middle();
                     if ( sound ) {
-                        globals->get_soundmgr()->add( sound, current_sound_name );
+                        _sgr->add( sound, current_sound_name );
                     }
                 }
             }
             if ( audio_btn->getBoolValue() ) {
-                if ( !globals->get_soundmgr()->is_playing(current_sound_name) ) {
-                    globals->get_soundmgr()->play_looped( current_sound_name );
+                if ( !_sgr->is_playing(current_sound_name) ) {
+                    _sgr->play_looped( current_sound_name );
                 }
             } else {
-                globals->get_soundmgr()->stop( current_sound_name );
+                _sgr->stop( current_sound_name );
             }
         } else if ( beacon_type == INNER ) {
             inner_marker = true;
             current_sound_name = "inner-marker";
             // cout << "INNER MARKER" << endl;
             if ( last_beacon != INNER ) {
-                if ( ! globals->get_soundmgr()->exists( current_sound_name ) ) {
+                if ( ! _sgr->exists( current_sound_name ) ) {
                     SGSoundSample *sound = beacon.get_inner();
                     if ( sound ) {
-                        globals->get_soundmgr()->add( sound, current_sound_name );
+                        _sgr->add( sound, current_sound_name );
                     }
                 }
             }
             if ( audio_btn->getBoolValue() ) {
-                if ( !globals->get_soundmgr()->is_playing(current_sound_name) ) {
-                    globals->get_soundmgr()->play_looped( current_sound_name );
+                if ( !_sgr->is_playing(current_sound_name) ) {
+                    _sgr->play_looped( current_sound_name );
                 }
             } else {
-                globals->get_soundmgr()->stop( current_sound_name );
+                _sgr->stop( current_sound_name );
             }
         }
         // cout << "VOLUME " << audio_vol->getDoubleValue() << endl;
-        SGSoundSample * mkr = globals->get_soundmgr()->find( current_sound_name );
+        SGSoundSample * mkr = _sgr->find( current_sound_name );
         if (mkr)
-            mkr->set_volume( audio_vol->getDoubleValue() );
+            mkr->set_volume( audio_vol->getFloatValue() );
     }
 
     if ( inrange ) {
