@@ -25,7 +25,12 @@
 #  include <config.h>
 #endif
 
-#if defined(__linux__) && defined(__i386__)
+#if defined(HAVE_FEENABLEEXCEPT)
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+#include <fenv.h>
+#elif defined(__linux__) && defined(__i386__)
 #  include <fpu_control.h>
 #endif
 
@@ -37,13 +42,17 @@
 #include <simgear/compiler.h>
 #include <simgear/structure/exception.hxx>
 #include <simgear/debug/logstream.hxx>
+#include <simgear/math/SGMath.hxx>
 
+#include <cstring>
 #include <iostream>
 using std::cerr;
 using std::endl;
 
 #include "main.hxx"
 #include "globals.hxx"
+#include "fg_props.hxx"
+#include "fgviewer.hxx"
 
 
 #include "fg_os.hxx"
@@ -55,9 +64,28 @@ bool free_hostname = false;
 // foreward declaration.
 void fgExitCleanup();
 
-#if defined(__linux__) && defined(__i386__)
+static bool fpeAbort = false;
+static void handleFPE(int);
+static void initFPE();
 
-static void handleFPE (int);
+#if defined(HAVE_FEENABLEEXCEPT)
+static void
+initFPE ()
+{
+    if (fpeAbort) {
+        int except = fegetexcept();
+        feenableexcept(except | FE_DIVBYZERO | FE_INVALID);
+    } else {
+        signal(SIGFPE, handleFPE);
+    }
+}
+
+static void handleFPE(int)
+{
+    feclearexcept(FE_ALL_EXCEPT);
+    signal(SIGFPE, handleFPE);
+}
+#elif defined(__linux__) && defined(__i386__)
 
 static void
 initFPE ()
@@ -80,6 +108,14 @@ handleFPE (int num)
   initFPE();
   SG_LOG(SG_GENERAL, SG_ALERT, "Floating point interrupt (SIGFPE)");
 }
+#else
+static void handleFPE(int)
+{
+}
+
+static void initFPE()
+{
+}
 #endif
 
 #ifdef _MSC_VER
@@ -134,14 +170,20 @@ int main ( int argc, char **argv ) {
 #endif
     _bootstrap_OSInit = 0;
 
-#if defined(__linux__) && defined(__i386__)
-    // Enable floating-point exceptions for Linux/x86
-    initFPE();
-#elif defined(__FreeBSD__)
+#if defined(__FreeBSD__)
     // Ignore floating-point exceptions on FreeBSD
-    signal(SIGFPE, SIG_IGN);
+    signal(SIGFPE, SIG_IGN); 
+#else
+    // Maybe Enable floating-point exceptions on Linux
+    for (int i = 0; i < argc; ++i) {
+        if (!strcmp("--enable-fpe", argv[i])) {
+            fpeAbort = true;
+            break;
+        }
+    }
+    initFPE();
 #endif
-#if !defined( _MSC_VER ) && !defined( __MINGW32__ )
+#ifndef _WIN32
     signal(SIGPIPE, SIG_IGN);
 #endif
 
@@ -168,19 +210,29 @@ int main ( int argc, char **argv ) {
 #if defined( HAVE_BC5PLUS )
     _control87(MCW_EM, MCW_EM);  /* defined in float.h */
 #endif
+    bool fgviewer = false;
+    for (int i = 0; i < argc; ++i) {
+        if (!strcmp("--fgviewer", argv[i])) {
+            fgviewer = true;
+            break;
+        }
+    }
 
     // FIXME: add other, more specific
     // exceptions.
     try {
         std::set_terminate(fg_terminate);
         atexit(fgExitCleanup);
-        fgMainInit(argc, argv);
+        if (fgviewer)
+            fgviewerMain(argc, argv);
+        else
+            fgMainInit(argc, argv);
     } catch (const sg_throwable &t) {
                             // We must use cerr rather than
                             // logging, since logging may be
                             // disabled.
         cerr << "Fatal error: " << t.getFormattedMessage() << endl;
-        if (!t.getOrigin().empty())
+        if (std::strlen(t.getOrigin()) != 0)
             cerr << " (received from " << t.getOrigin() << ')' << endl;
 
     } catch (const string &s) {
