@@ -15,16 +15,17 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: util.cxx,v 1.11.2.5 2007-11-27 19:28:29 durk Exp $
+// $Id$
 
 
 #include <simgear/compiler.h>
 
 #include <math.h>
-#include <stdlib.h>
+
+#include <cstdlib>
 
 #include <vector>
-SG_USING_STD(vector);
+using std::vector;
 
 #include <simgear/debug/logstream.hxx>
 
@@ -33,6 +34,9 @@ SG_USING_STD(vector);
 #include "globals.hxx"
 #include "util.hxx"
 
+#ifdef OSG_LIBRARY_STATIC
+#include "osgDB/Registry"
+#endif
 
 void
 fgDefaultWeatherValue (const char * propname, double value)
@@ -105,8 +109,12 @@ fgSetupWind (double min_hdg, double max_hdg, double speed, double gust)
 void
 fgExit (int status)
 {
+#ifdef OSG_LIBRARY_STATIC
+    osgDB::Registry::instance( true);
+#endif
+
     SG_LOG(SG_GENERAL, SG_INFO, "Exiting FlightGear with status " << status);
-    exit(status);
+    std::exit(status);
 }
 
 
@@ -139,7 +147,7 @@ fgGetLowPass (double current, double target, double timeratio)
 
 
 string
-fgUnescape(const char *s)
+fgUnescape (const char *s)
 {
     string r;
     while (*s) {
@@ -189,5 +197,27 @@ fgUnescape(const char *s)
     return r;
 }
 
+
+// Write out path to validation node and read it back in. A Nasal
+// listener is supposed to replace the path with a validated version
+// or an empty string otherwise.
+const char *fgValidatePath (const char *str, bool write)
+{
+    static SGPropertyNode_ptr r, w;
+    if (!r) {
+        r = fgGetNode("/sim/paths/validate/read", true);
+        r->setAttribute(SGPropertyNode::READ, true);
+        r->setAttribute(SGPropertyNode::WRITE, true);
+
+        w = fgGetNode("/sim/paths/validate/write", true);
+        w->setAttribute(SGPropertyNode::READ, true);
+        w->setAttribute(SGPropertyNode::WRITE, true);
+    }
+    SGPropertyNode *prop = write ? w : r;
+    prop->setStringValue(str);
+    const char *result = prop->getStringValue();
+    return result[0] ? result : 0;
+}
+
 // end of util.cxx
 
