@@ -465,13 +465,13 @@ int main( int argc, char **argv ) {
 
         current_time_stamp.stamp();
         /* Convert to ms */
-        double elapsed_us = current_time_stamp - last_time_stamp;
+        double elapsed_us = (current_time_stamp - last_time_stamp).toUSecs();
         if ( elapsed_us < (frame_us - 2000) ) {
             double requested_us = (frame_us - elapsed_us) - 2000 ;
             ulMilliSecondSleep ( (int)(requested_us / 1000.0) ) ;
         }
         current_time_stamp.stamp();
-        while ( current_time_stamp - last_time_stamp < frame_us ) {
+        while ( (current_time_stamp - last_time_stamp).toUSecs() < frame_us ) {
             current_time_stamp.stamp();
         }
 
