@@ -126,7 +126,7 @@ int main ( int argc, char ** argv )
 
     /* give other processes a chance */
 
-#ifdef WIN32
+#ifdef _WIN32
     Sleep ( 1 ) ;
 #elif defined(sgi)
     sginap ( 1 ) ;
